#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example demonstrates JMS notifications for completion of async operation
"""

import sys
from time import sleep
import threading
from pprint import pprint
import yaml
import requests.packages.urllib3
import stomp


import zhmcclient
__callback = None

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" %
          (hmccreds_file))
    sys.exit(1)

jms_notifications = examples.get("jms_notifications", None)
if jms_notifications is None:
    print("jms_notifications not found in credentials file %s" %
          (hmccreds_file))
    sys.exit(1)

hmc = jms_notifications["hmc"]
cpcname = jms_notifications["cpcname"]
partname = jms_notifications["partname"]
amqport = jms_notifications['amqport']
callback = None
topic = None

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" %
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

# Thread-safe handover of notifications between listener and main threads
NOTI_DATA = None
NOTI_LOCK = threading.Condition()

class MyListener(object):

    def on_connecting(self, host_and_port):
        print("Listener: Attempting to connect to message broker")
        sys.stdout.flush()

    def on_connected(self, headers, message):
        print("Listener: Connected to broker")
        sys.stdout.flush()

    def on_disconnected(self):
        print("Listener: Disconnected from broker")
        sys.stdout.flush()

    def on_error(self, headers, message):
        print('Listener: Received an error: %s' % message)
        sys.stdout.flush()

    def on_message(self, headers, message):
        global NOTI_DATA, NOTI_LOCK
        print('Listener: Received a notification')
        sys.stdout.flush()
        with NOTI_LOCK:
            # Wait until main program has processed the previous notification
            while NOTI_DATA:
                NOTI_LOCK.wait()
            # Indicate to main program that there is a new notification
            NOTI_DATA = headers
            NOTI_LOCK.notifyAll()

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

print("Retrieving notification topics ...")
topics = session.get_notification_topics()

for topic in topics:
    if topic['topic-type'] == 'job-notification':
        job_topic_name = topic['topic-name']
        break

conn = stomp.Connection([(session.host, amqport)], use_ssl="SSL")
conn.set_listener('', MyListener())
conn.start()
conn.connect(userid, password, wait=True)

sub_id = 42  # subscription ID

print("Subscribing for job notifications using topic: %s" % job_topic_name)
conn.subscribe(destination="/topic/"+job_topic_name, id=sub_id, ack='auto')

print("Finding CPC by name=%s ..." % cpcname)
try:
    cpc = cl.cpcs.find(name=cpcname)
except zhmcclient.NotFound:
    print("Could not find CPC %s on HMC %s" % (cpcname, hmc))
    sys.exit(1)

print("Finding partition by name=%s ..." % partname)
try:
    partition = cpc.partitions.find(name=partname)
except zhmcclient.NotFound:
    print("Could not find partition %s in CPC %s" % (partname, cpc.name))
    sys.exit(1)

print("Accessing status of partition %s ..." % partition.name)
partition_status = partition.get_property('status')
print("Status of partition %s: %s" % (partition.name, partition_status))

if partition_status == 'active':
    print("Stopping partition %s asynchronously ..." % partition.name)
    job = partition.stop(wait_for_completion=False)
elif partition_status in ('inactive', 'stopped'):
    print("Starting partition %s asynchronously ..." % partition.name)
    job = partition.start(wait_for_completion=False)
else:
    raise zhmcclient.Error("Cannot deal with partition status: %s" % \
                           partition_status)
print("Waiting for completion of job %s ..." % job.uri)
sys.stdout.flush()

# Just for demo purposes, we show how a loop for processing multiple
# notifications would look like.
while True:
    with NOTI_LOCK:

        # Wait until listener has a new notification
        while not NOTI_DATA:
            NOTI_LOCK.wait()

        # Process the notification
        print("Received notification:")
        pprint(NOTI_DATA)
        sys.stdout.flush()

        # This test is just for demo purposes, it should always be our job
        # given what we subscribed for.
        if NOTI_DATA['job-uri'] == job.uri:
            break
        else:
            print("Unexpected completion received for job %s" % \
                  NOTI_DATA['job-uri'])
            sys.stdout.flush()

        # Indicate to listener that we are ready for next notification
        NOTI_DATA = None
        NOTI_LOCK.notifyAll()

print("Job has completed: %s" % job.uri)
sys.stdout.flush()

conn.disconnect()
sleep(1)  # Allow listener to print disconnect message (just for demo)

print("Done.")



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows how to get partial and full properties for CPCs
and for LPARs of a CPC.
"""

import sys
import logging
import yaml
import requests.packages.urllib3
from datetime import datetime

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

get_partial_and_full_properties = examples.get("get_partial_and_full_properties", None)
if get_partial_and_full_properties is None:
    print("get_partial_and_full_properties not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = get_partial_and_full_properties.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = get_partial_and_full_properties["hmc"]
cpcname = get_partial_and_full_properties["cpcname"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)
timestats = get_partial_and_full_properties.get("timestats", None)
if timestats:
    session.time_stats_keeper.enable()

for full_properties in (False, True):
    print("Listing CPCs with full_properties=%s ..." % full_properties)
    start_dt = datetime.now()
    cpcs = cl.cpcs.list(full_properties)
    end_dt = datetime.now()
    duration = end_dt - start_dt
    print("Duration: %s" % duration)
    for cpc in cpcs:
        print("Number of properties of CPC %s: %s" %
              (cpc.name, len(cpc.properties)))

print("Finding CPC by name=%s ..." % cpcname)
try:
    cpc = cl.cpcs.find(name=cpcname)
except zhmcclient.NotFound:
    print("Could not find CPC %s on HMC %s" % (cpcname, hmc))
    sys.exit(1)
print("Found CPC %s at: %s" % (cpc.name, cpc.uri))

for full_properties in (False, True):
    print("Listing LPARs on CPC %s with full_properties=%s ..." %
          (cpc.name, full_properties))
    start_dt = datetime.now()
    lpars = cpc.lpars.list(full_properties)
    end_dt = datetime.now()
    duration = end_dt - start_dt
    print("Duration: %s" % duration)
    for lpar in lpars:
        print("Number of properties of LPAR %s: %s" %
              (lpar.name, len(lpar.properties)))

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A faked Session class for the zhmcclient package.
"""

from __future__ import absolute_import

import zhmcclient

from ._hmc import FakedHmc
from ._urihandler import UriHandler, HTTPError, URIS

__all__ = ['FakedSession']


class FakedSession(zhmcclient.Session):
    """
    A faked Session class for the zhmcclient package, that can be used as a
    replacement for the :class:`zhmcclient.Session` class.

    This class is derived from :class:`zhmcclient.Session`.

    This class can be used by projects using the zhmcclient package for their
    unit testing. It can also be used by unit tests of the zhmcclient package
    itself.

    This class provides a faked HMC with all of its resources that are relevant
    for the zhmcclient.

    The faked HMC provided by this class maintains its resource state in memory
    as Python objects, and no communication happens to any real HMC. The
    faked HMC implements all HMC operations that are relevant for the
    zhmcclient package in a successful manner.

    It is possible to populate the faked HMC with an initial resource state
    (see :meth:`~zhmcclient_mock.FakedHmc.add_resources`).
    """

    def __init__(self, host, hmc_name, hmc_version, api_version):
        """
        Parameters:

          host (:term:`string`):
            HMC host.

          hmc_name (:term:`string`):
            HMC name. Used for result of Query Version Info operation.

          hmc_version (:term:`string`):
            HMC version string (e.g. '2.13.1'). Used for result of
            Query Version Info operation.

          api_version (:term:`string`):
            HMC API version string (e.g. '1.8'). Used for result of
            Query Version Info operation.
        """
        super(FakedSession, self).__init__(host)
        self._hmc = FakedHmc(hmc_name, hmc_version, api_version)
        self._urihandler = UriHandler(URIS)

    @property
    def hmc(self):
        """
        :class:`~zhmcclient_mock.FakedHmc`: The faked HMC provided by this
        faked session.

        The faked HMC supports being populated with initial resource state,
        for example using its :meth:`zhmcclient_mock.FakedHmc.add_resources`
        method.

        As an alternative to providing an entire resource tree, the resources
        can also be added one by one, from top to bottom, using the
        :meth:`zhmcclient_mock.FakedBaseManager.add` methods of the
        respective managers (the top-level manager for CPCs can be accessed
        via ``hmc.cpcs``).
        """
        return self._hmc

    def get(self, uri, logon_required=True):
        """
        Perform the HTTP GET method against the resource identified by a URI,
        on the faked HMC.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g. "/api/session".
            This URI is relative to the base URL of the session (see
            the :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC.

            Because this is a faked HMC, this does not perform a real logon,
            but it is still used to update the state in the faked HMC.

        Returns:

          :term:`json object` with the operation result.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError` (not implemented)
          :exc:`~zhmcclient.AuthError` (not implemented)
          :exc:`~zhmcclient.ConnectionError` (not implemented)
        """
        try:
            return self._urihandler.get(self._hmc, uri, logon_required)
        except HTTPError as exc:
            raise zhmcclient.HTTPError(exc.response())

    def post(self, uri, body=None, logon_required=True,
             wait_for_completion=True, operation_timeout=None):
        """
        Perform the HTTP POST method against the resource identified by a URI,
        using a provided request body, on the faked HMC.

        HMC operations using HTTP POST are either synchronous or asynchronous.
        Asynchronous operations return the URI of an asynchronously executing
        job that can be queried for status and result.

        Examples for synchronous operations:

        * With no response body: "Logon", "Update CPC Properties"
        * With a response body: "Create Partition"

        Examples for asynchronous operations:

        * With no ``job-results`` field in the completed job status response:
          "Start Partition"
        * With a ``job-results`` field in the completed job status response
          (under certain conditions): "Activate a Blade", or "Set CPC Power
          Save"

        The `wait_for_completion` parameter of this method can be used to deal
        with asynchronous HMC operations in a synchronous way.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g. "/api/session".
            This URI is relative to the base URL of the session (see the
            :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          body (:term:`json object`):
            JSON object to be used as the HTTP request body (payload).
            `None` means the same as an empty dictionary, namely that no HTTP
            body is included in the request.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC. For example, the "Logon" operation does
            not require that.

            Because this is a faked HMC, this does not perform a real logon,
            but it is still used to update the state in the faked HMC.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested HMC operation, as follows:

            * If `True`, this method will wait for completion of the requested
              operation, regardless of whether the operation is synchronous or
              asynchronous.

              This will cause an additional entry in the time statistics to be
              created for the asynchronous operation and waiting for its
              completion. This entry will have a URI that is the targeted URI,
              appended with "+completion".

            * If `False`, this method will immediately return the result of the
              HTTP POST method, regardless of whether the operation is
              synchronous or asynchronous.

          operation_timeout (:term:`number`):
            Timeout in seconds, when waiting for completion of an asynchronous
            operation. The special value 0 means that no timeout is set. `None`
            means that the default async operation timeout of the session is
            used.

            For `wait_for_completion=True`, a
            :exc:`~zhmcclient.OperationTimeout` is raised when the timeout
            expires.

            For `wait_for_completion=False`, this parameter has no effect.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns a JSON object
            representing the response body of the synchronous operation, or the
            response body of the completed job that performed the asynchronous
            operation. If a synchronous operation has no response body, `None`
            is returned.

            If `wait_for_completion` is `False`, returns a JSON object
            representing the response body of the synchronous or asynchronous
            operation. In case of an asynchronous operation, the JSON object
            will have a member named ``job-uri``, whose value can be used with
            the :meth:`~zhmcclient.Session.query_job_status` method to
            determine the status of the job and the result of the original
            operation, once the job has completed.

            See the section in the :term:`HMC API` book about the specific HMC
            operation and about the 'Query Job Status' operation, for a
            description of the members of the returned JSON objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError` (not implemented)
          :exc:`~zhmcclient.AuthError` (not implemented)
          :exc:`~zhmcclient.ConnectionError` (not implemented)
        """
        try:
            return self._urihandler.post(self._hmc, uri, body, logon_required,
                                         wait_for_completion)
        except HTTPError as exc:
            raise zhmcclient.HTTPError(exc.response())

    def delete(self, uri, logon_required=True):
        """
        Perform the HTTP DELETE method against the resource identified by a
        URI, on the faked HMC.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g.
            "/api/session/{session-id}".
            This URI is relative to the base URL of the session (see
            the :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC. For example, for the logoff operation, it
            does not make sense to first log on.

            Because this is a faked HMC, this does not perform a real logon,
            but it is still used to update the state in the faked HMC.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError` (not implemented)
          :exc:`~zhmcclient.AuthError` (not implemented)
          :exc:`~zhmcclient.ConnectionError` (not implemented)
        """
        try:
            self._urihandler.delete(self._hmc, uri, logon_required)
        except HTTPError as exc:
            raise zhmcclient.HTTPError(exc.response())



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _lpar module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client


class LparTests(unittest.TestCase):
    """All tests for Lpar and LparManager classes."""

    def setUp(self):
        self.session = Session('fake-host', 'fake-user', 'fake-id')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'fake-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/fake-cpc-id-1',
                        'name': 'P0ZHMP02',
                        'status': 'service-required',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)
            cpcs = self.cpc_mgr.list(full_properties=False)
            self.cpc = cpcs[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on LparManager instance in CPC."""
        lpar_mgr = self.cpc.lpars
        self.assertEqual(lpar_mgr.cpc, self.cpc)

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties on LparManager
        instance in CPC.
        """
        lpar_mgr = self.cpc.lpars
        with requests_mock.mock() as m:
            result = {
                'logical-partitions': [
                    {
                        'status': 'not-activated',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                        'name': 'LPAR1'
                    },
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                        'name': 'LPAR2'
                    }
                ]
            }
            m.get('/api/cpcs/fake-cpc-id-1/logical-partitions', json=result)

            lpars = lpar_mgr.list(full_properties=False)

            self.assertEqual(len(lpars), len(result['logical-partitions']))
            for idx, lpar in enumerate(lpars):
                self.assertEqual(
                    lpar.properties,
                    result['logical-partitions'][idx])
                self.assertEqual(
                    lpar.uri,
                    result['logical-partitions'][idx]['object-uri'])
                self.assertFalse(lpar.full_properties)
                self.assertEqual(lpar.manager, lpar_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on LparManager
        instance in CPC.
        """
        lpar_mgr = self.cpc.lpars
        with requests_mock.mock() as m:
            result = {
                'logical-partitions': [
                    {
                        'status': 'not-activated',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                        'name': 'LPAR1'
                    },
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                        'name': 'LPAR2'
                    }

                ]
            }
            m.get('/api/cpcs/fake-cpc-id-1/logical-partitions', json=result)

            mock_result_lpar1 = {
                'status': 'not-activated',
                'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                'name': 'LPAR1',
                'description': 'LPAR Image',
                'more_properties': 'bliblablub'
            }
            m.get('/api/logical-partitions/fake-lpar-id-1',
                  json=mock_result_lpar1)
            mock_result_lpar2 = {
                'status': 'not-activated',
                'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                'name': 'LPAR2',
                'description': 'LPAR Image',
                'more_properties': 'bliblablub'
            }
            m.get('/api/logical-partitions/fake-lpar-id-2',
                  json=mock_result_lpar2)

            lpars = lpar_mgr.list(full_properties=True)

            self.assertEqual(len(lpars), len(result['logical-partitions']))
            for idx, lpar in enumerate(lpars):
                self.assertEqual(lpar.properties['name'],
                                 result['logical-partitions'][idx]['name'])
                self.assertEqual(
                    lpar.uri,
                    result['logical-partitions'][idx]['object-uri'])
                self.assertTrue(lpar.full_properties)
                self.assertEqual(lpar.manager, lpar_mgr)

    def test_activate(self):
        """
        This tests the 'Activate Logical Partition' operation.
        """
        lpar_mgr = self.cpc.lpars
        with requests_mock.mock() as m:
            result = {
                'logical-partitions': [
                    {
                        'status': 'not-activated',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                        'name': 'LPAR1'
                    },
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                        'name': 'LPAR2'
                    }
                ]
            }
            m.get('/api/cpcs/fake-cpc-id-1/logical-partitions', json=result)

            lpars = lpar_mgr.list(full_properties=False)
            lpar = lpars[0]
            result = {
                "job-reason-code": 0,
                "job-status-code": 204,
                "status": "complete"
            }
            m.post(
                "/api/logical-partitions/fake-lpar-id-1/operations/activate",
                json=result)
            status = lpar.activate(wait_for_completion=False)
            self.assertEqual(status, result)

    def test_deactivate(self):
        """
        This tests the 'Deactivate Logical Partition' operation.
        """
        lpar_mgr = self.cpc.lpars
        with requests_mock.mock() as m:
            result = {
                'logical-partitions': [
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                        'name': 'LPAR1'
                    },
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                        'name': 'LPAR2'
                    }
                ]
            }
            m.get('/api/cpcs/fake-cpc-id-1/logical-partitions', json=result)

            lpars = lpar_mgr.list(full_properties=False)
            lpar = lpars[0]
            result = {
                "job-reason-code": 0,
                "job-status-code": 204,
                "status": "complete"
            }
            m.post(
                "/api/logical-partitions/fake-lpar-id-1/operations/deactivate",
                json=result)
            status = lpar.deactivate(wait_for_completion=False)
            self.assertEqual(status, result)

    def test_load(self):
        """
        This tests the 'Load Logical Partition' operation.
        """
        lpar_mgr = self.cpc.lpars
        with requests_mock.mock() as m:
            result = {
                'logical-partitions': [
                    {
                        'status': 'not-operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-1',
                        'name': 'LPAR1'
                    },
                    {
                        'status': 'operating',
                        'object-uri': '/api/logical-partitions/fake-lpar-id-2',
                        'name': 'LPAR2'
                    }
                ]
            }
            m.get('/api/cpcs/fake-cpc-id-1/logical-partitions', json=result)

            lpars = lpar_mgr.list(full_properties=False)
            lpar = lpars[0]
            result = {
                "job-reason-code": 0,
                "job-status-code": 204,
                "status": "complete"
            }

            m.post("/api/logical-partitions/fake-lpar-id-1/operations/load",
                   json=result)
            status = lpar.load(load_address='5162', wait_for_completion=False)
            self.assertEqual(status, result)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`NIC` (Network Interface Card) is a logical entity that provides a
:term:`Partition` with access to external communication networks through a
:term:`Network Adapter`. More specifically, a NIC connects a Partition with a
:term:`Network Port`, or with a :term:`Virtual Switch` which then connects to
the Network Port.

NIC resources are contained in Partition resources.

NICs only exist in :term:`CPCs <CPC>` that are in DPM mode.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._logging import get_logger, logged_api_call

__all__ = ['NicManager', 'Nic']

LOG = get_logger(__name__)


class NicManager(BaseManager):
    """
    Manager providing access to the :term:`NICs <NIC>` in a particular
    :term:`Partition`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible via the following instance variable of a
    :class:`~zhmcclient.Partition` object (in DPM mode):

    * :attr:`~zhmcclient.Partition.nics`
    """

    def __init__(self, partition):
        # This function should not go into the docs.
        # Parameters:
        #   partition (:class:`~zhmcclient.Partition`):
        #     Partition defining the scope for this manager.

        super(NicManager, self).__init__(
            resource_class=Nic,
            parent=partition,
            uri_prop='element-uri',
            name_prop='name',
            query_props=[])

    @property
    def partition(self):
        """
        :class:`~zhmcclient.Partition`: :term:`Partition` defining the scope
        for this manager.
        """
        return self._parent

    @logged_api_call
    def list(self, full_properties=False, filter_args=None):
        """
        List the NICs in this Partition.

        Authorization requirements:

        * Object-access permission to this Partition.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          : A list of :class:`~zhmcclient.Nic` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        resource_obj_list = []
        uris = self.partition.get_property('nic-uris')
        if uris:
            for uri in uris:

                resource_obj = self.resource_class(
                    manager=self,
                    uri=uri,
                    name=None,
                    properties=None)

                if self._matches_filters(resource_obj, filter_args):
                    resource_obj_list.append(resource_obj)
                    if full_properties:
                        resource_obj.pull_full_properties()
        return resource_obj_list

    @logged_api_call
    def create(self, properties):
        """
        Create and configure a NIC in this Partition.

        Authorization requirements:

        * Object-access permission to this Partition.
        * Object-access permission to the backing Adapter for the new NIC.
        * Task permission to the "Partition Details" task.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create NIC' in the :term:`HMC API` book.

            The backing Adapter for the new NIC is identified as follows:

            * For OSA and Hipersockets adapters, the Adapter associated with
              the Virtual Switch designated by the "virtual-switch-uri"
              property.
            * For RoCE adapters, the Adapter of the Port designated by the
              "network-adapter-port-uri" property.

        Returns:

          Nic:
            The resource object for the new NIC.
            The object will have its 'element-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        result = self.session.post(self.partition.uri + '/nics',
                                   body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return Nic(self, props['element-uri'], None, props)

    @logged_api_call
    def nic_object(self, nic_id):
        """
        Return a minimalistic :class:`~zhmcclient.Nic` object for a Nic in this
        Partition.

        This method is an internal helper function and is not normally called
        by users.

        This object will be connected in the Python object tree representing
        the resources (i.e. it has this Partition as a parent), and will have
        the following properties set:

          * `element-uri`
          * `element-id`
          * `parent`
          * `class`

        Parameters:

            nic_id (string): `element-id` of the Nic

        Returns:

            :class:`~zhmcclient.Nic`: A Python object representing the Nic.
        """
        part_uri = self.parent.uri
        nic_uri = part_uri + "/nics/" + nic_id
        nic_props = {
            'element-id': nic_id,
            'parent': part_uri,
            'class': 'nic',
        }
        return Nic(self, nic_uri, None, nic_props)


class Nic(BaseResource):
    """
    Representation of a :term:`NIC`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of a NIC resource, see section
    'Data model - NIC Element Object' in section 'Partition object' in the
    :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.NicManager`).
    """

    def __init__(self, manager, uri, name=None, properties=None):
        # This function should not go into the docs.
        #   manager (:class:`~zhmcclient.NicManager`):
        #     Manager object for this resource object.
        #   uri (string):
        #     Canonical URI path of the resource.
        #   name (string):
        #     Name of the resource.
        #   properties (dict):
        #     Properties to be set for this resource object. May be `None` or
        #     empty.
        if not isinstance(manager, NicManager):
            raise AssertionError("Nic init: Expected manager type %s, got %s" %
                                 (NicManager, type(manager)))
        super(Nic, self).__init__(manager, uri, name, properties)

    @logged_api_call
    def delete(self):
        """
        Delete this NIC.

        Authorization requirements:

        * Object-access permission to the Partition containing this HBA.
        * Task permission to the "Partition Details" task.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.delete(self._uri)

    @logged_api_call
    def update_properties(self, properties):
        """
        Update writeable properties of this NIC.

        Authorization requirements:

        * Object-access permission to the Partition containing this NIC.
        * Object-access permission to the backing Adapter for this NIC.
        * Task permission to the "Partition Details" task.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model - NIC Element Object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.post(self._uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import click

import zhmcclient
from .zhmccli import cli
from ._helper import print_properties, print_resources, abort_if_false, \
    options_to_properties, original_options, COMMAND_OPTIONS_METAVAR
from ._cmd_cpc import find_cpc


def find_adapter(client, cpc_name, adapter_name):
    """
    Find an adapter by name and return its resource object.
    """
    cpc = find_cpc(client, cpc_name)
    # The CPC must be in DPM mode. We don't check that because it would
    # cause a GET to the CPC resource that we otherwise don't need.
    try:
        adapter = cpc.adapters.find(name=adapter_name)
    except zhmcclient.NotFound:
        raise click.ClickException("Could not find adapter %s in CPC %s." %
                                   (adapter_name, cpc_name))
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))
    return adapter


@cli.group('adapter', options_metavar=COMMAND_OPTIONS_METAVAR)
def adapter_group():
    """
    Command group for managing adapters.

    Physical adapters (e.g. OSA, FICON) are auto-discovered and cannot be
    created. Logical adapters (HiperSockets) can be created and deleted.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.
    """


@adapter_group.command('list', options_metavar=COMMAND_OPTIONS_METAVAR)
@click.argument('CPC', type=str, metavar='CPC')
@click.option('--type', is_flag=True, required=False,
              help='Show additional properties for the adapter family and '
              'type.')
@click.option('--uri', is_flag=True, required=False,
              help='Show additional properties for the resource URI.')
@click.pass_obj
def adapter_list(cmd_ctx, cpc, **options):
    """
    List the adapters in a CPC.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.
    """
    cmd_ctx.execute_cmd(lambda: cmd_adapter_list(cmd_ctx, cpc, options))


@adapter_group.command('show', options_metavar=COMMAND_OPTIONS_METAVAR)
@click.argument('CPC', type=str, metavar='CPC')
@click.argument('ADAPTER', type=str, metavar='ADAPTER')
@click.pass_obj
def adapter_show(cmd_ctx, cpc, adapter):
    """
    Show the details of an adapter.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.
    """
    cmd_ctx.execute_cmd(lambda: cmd_adapter_show(cmd_ctx, cpc, adapter))


@adapter_group.command('update', options_metavar=COMMAND_OPTIONS_METAVAR)
@click.argument('CPC', type=str, metavar='CPC')
@click.argument('ADAPTER', type=str, metavar='ADAPTER')
@click.option('--name', type=str, required=False,
              help='The new name of the adapter.')
@click.option('--description', type=str, required=False,
              help='The new description of the adapter.')
@click.option('--port-description', type=str, required=False,
              help='The new description of the single port of the adapter.')
@click.option('--mtu-size', type=click.Choice(['8', '16', '32', '56']),
              required=False,
              help='The new MTU size of the adapter in KiB. '
              '(HiperSockets only).')
@click.option('--allowed-capacity', type=int, required=False,
              help='The maximum number of HBAs per partition. '
              '(FCP only).')
@click.option('--chpid', type=str, required=False,
              help='Channel path ID (CHPID, 2 hex chars) used by the '
              'adapter\'s port. '
              '(OSA, FICON, HiperSockets only).')
@click.option('--crypto-number', type=int, required=False,
              help='Identifier of the crypto adapter in the range 0-15 '
              'Must be unique within the CPC. '
              '(Crypto only).')
@click.option('--crypto-tke/--no-crypto-tke', default=None, required=False,
              help='Permit TKE commands on the crypto adapter. '
              '(Crypto only).')
@click.pass_obj
def adapter_update(cmd_ctx, cpc, adapter, **options):
    """
    Update the properties of an adapter.

    Only the properties will be changed for which a corresponding option is
    specified, so the default for all options is not to change properties.

    The adapter may be a physical adapter (e.g. a discovered OSA card) or a
    logical adapter (e.g. HiperSockets).

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.
    """
    cmd_ctx.execute_cmd(lambda: cmd_adapter_update(cmd_ctx, cpc, adapter,
                                                   options))


@adapter_group.command('create', options_metavar=COMMAND_OPTIONS_METAVAR)
@click.argument('CPC', type=str, metavar='CPC')
@click.option('--name', type=str, required=True,
              help='The name of the new adapter.')
@click.option('--description', type=str, required=False,
              help='The description of the new adapter.')
@click.option('--port-description', type=str, required=False,
              help='The description of the (single) port of the new adapter.')
@click.option('--mtu-size', type=click.Choice(['8', '16', '32', '56']),
              required=False,
              help='The MTU size of the new adapter in KiB.')
@click.pass_obj
def adapter_create_hipersocket(cmd_ctx, cpc, **options):
    """
    Create a HiperSockets adapter in a CPC.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.

    Some more properties of the new HiperSockets adapter can be set via
    adapter update.
    """
    cmd_ctx.execute_cmd(lambda: cmd_adapter_create_hipersocket(
                        cmd_ctx, cpc, options))


@adapter_group.command('delete', options_metavar=COMMAND_OPTIONS_METAVAR)
@click.argument('CPC', type=str, metavar='CPC')
@click.argument('ADAPTER', type=str, metavar='ADAPTER')
@click.option('-y', '--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              help='Skip prompt to confirm deletion of the adapter.',
              prompt='Are you sure you want to delete this adapter ?')
@click.pass_obj
def adapter_delete_hipersocket(cmd_ctx, cpc, adapter):
    """
    Delete a HiperSockets adapter.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified right after the
    'zhmc' command name.
    """
    cmd_ctx.execute_cmd(lambda: cmd_adapter_delete_hipersocket(
                        cmd_ctx, cpc, adapter))


def cmd_adapter_list(cmd_ctx, cpc_name, options):

    client = zhmcclient.Client(cmd_ctx.session)
    cpc = find_cpc(client, cpc_name)

    try:
        adapters = cpc.adapters.list()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    show_list = [
        'name',
        'adapter-id',
        'status',
    ]
    if options['type']:
        show_list.extend([
            'adapter-family',
            'type',
            'detected-card-type',
        ])
    if options['uri']:
        show_list.extend([
            'object-uri',
        ])

    cmd_ctx.spinner.stop()
    print_resources(adapters, cmd_ctx.output_format, show_list)


def cmd_adapter_show(cmd_ctx, cpc_name, adapter_name):

    client = zhmcclient.Client(cmd_ctx.session)
    adapter = find_adapter(client, cpc_name, adapter_name)

    try:
        adapter.pull_full_properties()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    print_properties(adapter.properties, cmd_ctx.output_format)


def cmd_adapter_update(cmd_ctx, cpc_name, adapter_name, options):

    client = zhmcclient.Client(cmd_ctx.session)
    adapter = find_adapter(client, cpc_name, adapter_name)

    name_map = {
        'mtu-size': 'maximum-transmission-unit-size',
        'chpid': 'channel-path-id',
        'crypto-tke': 'tke-commands-enabled',
    }
    options = original_options(options)
    properties = options_to_properties(options, name_map)

    if not properties:
        cmd_ctx.spinner.stop()
        click.echo("No properties specified for updating adapter %s." %
                   adapter_name)
        return

    try:
        adapter.update_properties(properties)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    if 'name' in properties and properties['name'] != adapter_name:
        click.echo("Adapter %s has been renamed to %s and was updated." %
                   (adapter_name, properties['name']))
    else:
        click.echo("Adapter %s has been updated." % adapter_name)


def cmd_adapter_create_hipersocket(cmd_ctx, cpc_name, options):

    client = zhmcclient.Client(cmd_ctx.session)
    cpc = find_cpc(client, cpc_name)

    name_map = {
        'mtu-size': 'maximum-transmission-unit-size',
    }
    options = original_options(options)
    properties = options_to_properties(options, name_map)

    try:
        new_adapter = cpc.adapters.create_hipersocket(properties)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    click.echo("New HiperSockets adapter %s has been created." %
               new_adapter.properties['name'])


def cmd_adapter_delete_hipersocket(cmd_ctx, cpc_name, adapter_name):

    client = zhmcclient.Client(cmd_ctx.session)
    adapter = find_adapter(client, cpc_name, adapter_name)

    try:
        adapter.delete()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    click.echo('HiperSockets adapter %s has been deleted.' % adapter_name)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows the API version of an HMC.
"""

import sys
import logging
import yaml
import requests.packages.urllib3

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

api_version = examples.get("api_version", None)
if api_version is None:
    print("api_version not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = api_version.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = api_version["hmc"]

print(__doc__)

print("Using HMC %s with an unauthenticated session ..." % hmc)
session = zhmcclient.Session(hmc)
cl = zhmcclient.Client(session)

vi = cl.version_info()
print("HMC API version: {}.{}".format(vi[0], vi[1]))



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _activation_profile module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client


class ActivationProfileTests(unittest.TestCase):
    """All tests for ActivationProfile and ActvationProfileManager classes."""

    def setUp(self):
        self.session = Session('ap-test-host', 'ap-test-user', 'ap-test-id')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'ap-test-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/ap-test-cpc-id-1',
                        'name': 'APCPC1',
                        'status': 'service-required',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)
            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on ActivationManager instance in CPC."""
        reset_ap_mgr = self.cpc.reset_activation_profiles
        self.assertEqual(reset_ap_mgr.cpc, self.cpc)
        self.assertEqual(reset_ap_mgr.profile_type, "reset")

        image_ap_mgr = self.cpc.image_activation_profiles
        self.assertEqual(image_ap_mgr.cpc, self.cpc)
        self.assertEqual(image_ap_mgr.profile_type, "image")

        load_ap_mgr = self.cpc.load_activation_profiles
        self.assertEqual(load_ap_mgr.cpc, self.cpc)
        self.assertEqual(load_ap_mgr.profile_type, "load")

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties
        on ActivationProfileManager instance in CPC.
        """
        image_mgr = self.cpc.image_activation_profiles
        with requests_mock.mock() as m:
            result = {
                'image-activation-profiles': [
                    {
                        'name': 'LPAR1',
                        'element-uri': '/api/cpcs/fake-element-uri-id-1/'
                                       'image-activation-profiles/LPAR1'
                    },
                    {
                        'name': 'LPAR2',
                        'element-uri': '/api/cpcs/fake-element-uri-id-2/'
                                       'image-activation-profiles/LPAR2'
                    }
                ]
            }

            m.get('/api/cpcs/ap-test-cpc-id-1/image-activation-profiles',
                  json=result)

            profiles = image_mgr.list(full_properties=False)

            self.assertEqual(len(profiles),
                             len(result['image-activation-profiles']))
            for idx, profile in enumerate(profiles):
                self.assertEqual(
                    profile.properties,
                    result['image-activation-profiles'][idx])
                self.assertEqual(
                    profile.uri,
                    result['image-activation-profiles'][idx]['element-uri'])
                self.assertFalse(profile.full_properties)
                self.assertEqual(profile.manager, image_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties
        on ActivationProfileManager instance in CPC.
        """
        image_mgr = self.cpc.image_activation_profiles
        with requests_mock.mock() as m:
            result = {
                'image-activation-profiles': [
                    {
                        'name': 'LPAR1',
                        'element-uri': '/api/cpcs/fake-element-uri-id-1/'
                                       'image-activation-profiles/LPAR1'
                    },
                    {
                        'name': 'LPAR2',
                        'element-uri': '/api/cpcs/fake-element-uri-id-2/'
                                       'image-activation-profiles/LPAR2'
                    }
                ]
            }

            m.get('/api/cpcs/ap-test-cpc-id-1/image-activation-profiles',
                  json=result)

            mock_result_part1 = {
                'element-uri': '/api/cpcs/fake-element-uri-id-1/'
                               'image-activation-profiles/LPAR1',
                'name': 'LPAR1',
                'description': 'Image Activation Profile',
                'more_properties': 'bliblablub'
            }
            m.get('/api/cpcs/fake-element-uri-id-1/image-activation-profiles/'
                  'LPAR1',
                  json=mock_result_part1)
            mock_result_part2 = {
                'element-uri': '/api/cpcs/fake-element-uri-id-2/'
                               'image-activation-profiles/LPAR2',
                'name': 'LPAR2',
                'description': 'Image Activation Profile',
                'more_properties': 'bliblablub'
            }
            m.get('/api/cpcs/fake-element-uri-id-2/image-activation-profiles/'
                  'LPAR2',
                  json=mock_result_part2)

            profiles = image_mgr.list(full_properties=True)

            self.assertEqual(len(profiles),
                             len(result['image-activation-profiles']))
            for idx, profile in enumerate(profiles):
                self.assertEqual(
                    profile.properties['name'],
                    result['image-activation-profiles'][idx]['name'])
                self.assertEqual(
                    profile.uri,
                    result['image-activation-profiles'][idx]['element-uri'])
                self.assertTrue(profile.full_properties)
                self.assertEqual(profile.manager, image_mgr)

    def test_update_properties(self):
        """
        This tests the 'Update Activation Profile Properties' operation.
        """
        image_mgr = self.cpc.image_activation_profiles
        with requests_mock.mock() as m:
            result = {
                'image-activation-profiles': [
                    {
                        'name': 'LPAR1',
                        'element-uri': '/api/cpcs/fake-element-uri-id-1/'
                                       'image-activation-profiles/LPAR1'
                    },
                    {
                        'name': 'LPAR2',
                        'element-uri': '/api/cpcs/fake-element-uri-id-2/'
                                       'image-activation-profiles/LPAR2'
                    }
                ]
            }

            m.get('/api/cpcs/ap-test-cpc-id-1/image-activation-profiles',
                  json=result)
            profiles = image_mgr.list(full_properties=False)
            profile = profiles[0]
            m.post(
                '/api/cpcs/fake-element-uri-id-1/image-activation-profiles/'
                'LPAR1',
                json=result)
            status = profile.update_properties(properties={})
            self.assertEqual(status, None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows how to use the asynchronous interface in polling mode.
"""

import sys
import logging
import yaml
import requests.packages.urllib3
import time

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

async_operation_polling = examples.get("async_operation_polling", None)
if async_operation_polling is None:
    print("async_operation_polling not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = async_operation_polling.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = async_operation_polling["hmc"]
cpcname = async_operation_polling["cpcname"]
cpcstatus = async_operation_polling["cpcstatus"]
lparname = async_operation_polling["lparname"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

timestats = async_operation_polling.get("timestats", None)
if timestats:
    session.time_stats_keeper.enable()

print("Finding CPC by name=%s and status=%s ..." % (cpcname, cpcstatus))
try:
    cpc = cl.cpcs.find(name=cpcname, status=cpcstatus)
except zhmcclient.NotFound:
    print("Could not find CPC %s with status %s on HMC %s" %
          (cpcname, cpcstatus, hmc))
    sys.exit(1)

print("Finding LPAR by name=%s ..." % lparname)
try:
    lpar = cpc.lpars.find(name=lparname)
except zhmcclient.NotFound:
    print("Could not find LPAR %s in CPC %s" % (lparname, cpc.name))
    sys.exit(1)

print("Accessing status of LPAR %s ..." % lpar.name)
status = lpar.get_property('status')
print("Status of LPAR %s: %s" % (lpar.name, status))

print("De-Activating LPAR %s (async.) ..." % lpar.name)
job = lpar.deactivate(wait_for_completion=False)
while True:
    print("Retrieving job status ...")
    job_status, _ = job.check_for_completion()
    print("Job status: %s" % job_status)
    if job_status == 'complete':
        break
    time.sleep(1)
print('De-Activation complete!')

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



#!/usr/bin/env python
# Copyright 2017 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The HMC supports the publishing of notifications for specific topics. This
includes for example asynchronous job completion, property or status changes,
and operating system messages issued in an LPAR or DPM partition.

The zhmcclient package supports receiving HMC notifications in an easy-to-use
way, as shown in the following example that receives and displays OS messages
for a DPM partition::

    import zhmcclient

    hmc = ...
    userid = ...
    password = ...

    session = zhmcclient.Session(hmc, userid, password)
    client = zhmcclient.Client(session)
    cpc = client.cpcs.find(name=cpcname)
    partition = cpc.partitions.find(name=partname)

    topic = partition.open_os_message_channel(include_refresh_messages=True)

    print("Subscribing for OS messages for partition %s on CPC %s..." %
          (partition.name, cpc.name))

    receiver = zhmcclient.NotificationReceiver(topic, hmc, userid, password)

    try:
        for headers, message in receiver.notifications():
            print("HMC notification #%s:" % headers['session-sequence-nr'])
            os_msg_list = message['os-messages']
            for os_msg in os_msg_list:
                msg_txt = os_msg['message-text'].strip('\\n')
                msg_id = os_msg['message-id']
                print("OS message #%s:\\n%s" % (msg_id, msg_txt))
    except KeyboardInterrupt:
        pass

    print("Closing OS message channel...")
    receiver.close()

When running this example code in one terminal, and stopping or starting
the partition in another terminal, one can monitor the shutdown or boot
messages issued by the operating system. The following commands use the
CLI of the zhmcclient package to do that::

    $ zhmc partition stop <cpc-name> <partition-name>
    $ zhmc partition start <cpc-name> <partition-name>
"""

import threading
import stomp
import json

from ._logging import get_logger, logged_api_call

__all__ = ['NotificationReceiver']

LOG = get_logger(__name__)


# Port on which the HMC issues JMS over STOMP messages:
_STOMP_PORT = 61612


class NotificationReceiver(object):
    """
    A class for receiving HMC notifications that are published to a particular
    single HMC notification topic.

    **Experimental:** This class is considered experimental at this point, and
    its API may change incompatibly as long as it is experimental.

    Creating an object of this class establishes a JMS session with the
    HMC and subscribes for a particular HMC notification topic.

    Notification topic strings are created by the HMC in context of a
    particular client session (i.e. :class:`~zhmcclient.Session` object).
    However, these topic strings can be used by any JMS message listener that
    knows the topic string and that authenticates under some valid HMC userid.
    The HMC userid used by the JMS listener does not need to be the one that
    was used for the client session in which the notification topic was
    originally created.
    """

    def __init__(self, topic, host, userid, password):
        """
        Parameters:

          topic (:term:`string`): Name of the HMC notification topic.
            Must not be `None`.

          host (:term:`string`):
            HMC host. For valid formats, see the
            :attr:`~zhmcclient.Session.host` property.
            Must not be `None`.

          userid (:term:`string`):
            Userid of the HMC user to be used.
            Must not be `None`.

          password (:term:`string`):
            Password of the HMC user to be used.
            Must not be `None`.
        """
        self._topic = topic
        self._host = host
        self._userid = userid
        self._password = password

        # Wait timeout to honor keyboard interrupts after this time:
        self._wait_timeout = 10.0  # seconds

        # Subscription ID. We use some value that allows to identify on the
        # HMC that this is the zhmcclient, but otherwise we are not using
        # this value ourselves.
        self._sub_id = 'zhmcclient.%s' % id(self)

        # Sync variables for thread-safe handover between listener thread and
        # receiver thread:
        self._handover_dict = {}
        self._handover_cond = threading.Condition()

        self._conn = stomp.Connection(
            [(self._host, _STOMP_PORT)], use_ssl="SSL")
        listener = _NotificationListener(self._handover_dict,
                                         self._handover_cond)
        self._conn.set_listener('', listener)
        self._conn.start()
        self._conn.connect(self._userid, self._password, wait=True)

        dest = "/topic/" + topic
        self._conn.subscribe(destination=dest, id=self._sub_id, ack='auto')

    @logged_api_call
    def notifications(self):
        """
        Generator method that yields all HMC notifications (= JMS messages)
        received by this notification receiver.

        Example::

            receiver = zhmcclient.NotificationReceiver(topic, hmc, userid,
                                                       password)
            for headers, message in receiver.notifications():
                . . .

        Yields:

          : A tuple (headers, message) representing one HMC notification, with:

          * headers (dict): The notification header fields.

            Some important header fields (dict items) are:

            * 'notification-type' (string): The HMC notification type (e.g.
              'os-message', 'job-completion', or others).

            * 'session-sequence-nr' (string): The sequence number of this HMC
              notification within the session created by this notification
              receiver object. This number starts at 0 when this receiver
              object is created, and is incremented each time an HMC
              notification is published to this receiver.

            * 'class' (string): The class name of the HMC resource publishing
              the HMC notification (e.g. 'partition').

            * 'object-id' (string) or 'element-id' (string): The ID of the HMC
              resource publishing the HMC notification.

            For a complete list of notification header fields, see section
            "Message format" in chapter 4. "Asynchronous notification" in the
            :term:`HMC API` book.

          * message (:term:`JSON object`): Body of the HMC notification,
            converted into a JSON object.

            The properties of the JSON object vary by notification type.

            For a description of the JSON properties, see the sub-sections
            for each notification type within section "Notification message
            formats" in chapter 4. "Asynchronous notification" in the
            :term:`HMC API` book.
        """

        while True:
            with self._handover_cond:

                # Wait until MessageListener has a new notification
                while len(self._handover_dict) == 0:
                    self._handover_cond.wait(self._wait_timeout)

                if self._handover_dict['headers'] is None:
                    return

                # Process the notification
                yield (self._handover_dict['headers'],
                       self._handover_dict['message'])

                # Indicate to MessageListener that we are ready for next
                # notification
                del self._handover_dict['headers']
                del self._handover_dict['message']
                self._handover_cond.notifyAll()

    @logged_api_call
    def close(self):
        """
        Disconnect and close the JMS session with the HMC.

        This implicitly unsubscribes from the notification topic this receiver
        was created for, and causes the
        :meth:`~zhmcclient.NotificationReceiver.notifications` method to
        stop its iterations.
        """
        self._conn.disconnect()


class _NotificationListener(object):
    """
    A notification listener class for use by the Python `stomp` package.

    This is an internal class that does not need to be accessed or created by
    the user. An object of this class is automatically created by the
    :class:`~zhmcclient.NotificationReceiver` class, for its notification
    topic.
    """

    def __init__(self, handover_dict, handover_cond):
        """
        Parameters:

          handover_dict (dict): Dictionary for handing over the notification
            header and message from this listener thread to the receiver
            thread. Must initially be an empty dictionary.

          handover_cond (threading.Condition): Condition object for handing
            over the notification from this listener thread to the receiver
            thread. Must initially be a new threading.Condition object.
        """

        # Sync variables for thread-safe handover between listener thread and
        # receiver thread:
        self._handover_dict = handover_dict  # keys: headers, message
        self._handover_cond = handover_cond

        # Wait timeout to honor keyboard interrupts after this time:
        self._wait_timeout = 10.0  # seconds

    def on_disconnected(self):
        """
        Event method that gets called when the JMS session has been
        disconnected.

        It hands over a termination notification (headers and message are
        None).
        """

        with self._handover_cond:

            # Wait until receiver has processed the previous notification
            while len(self._handover_dict) > 0:
                self._handover_cond.wait(self._wait_timeout)

            # Indicate to receiver that there is a termination notification
            self._handover_dict['headers'] = None  # terminate receiver
            self._handover_dict['message'] = None
            self._handover_cond.notifyAll()

    def on_error(self, headers, message):
        """
        This event method should never be called, because the HMC does not
        issue JMS errors.

        For parameters, see
        :meth:`~zhmcclient.NotificationListener.on_message`.
        """
        raise RuntimeError("Unexpectedly received a JMS error "
                           "(JMS headers: %r)" % headers)

    def on_message(self, headers, message):
        """
        Event method that gets called when this listener has received a JMS
        message (representing an HMC notification).

        Parameters:

          headers (dict): JMS message headers, as described for `headers` tuple
            item returned by the
            :meth:`~zhmcclient.NotificationReceiver.notifications` method.

          message (string): JMS message body as a string, which contains a
            serialized JSON object. The JSON object is described in the
            `message` tuple item returned by the
            :meth:`~zhmcclient.NotificationReceiver.notifications` method).
        """

        with self._handover_cond:

            # Wait until receiver has processed the previous notification
            while len(self._handover_dict) > 0:
                self._handover_cond.wait(self._wait_timeout)

            # Indicate to receiver that there is a new notification
            self._handover_dict['headers'] = headers
            try:
                msg_obj = json.loads(message)
            except Exception:
                raise  # TODO: Find better exception for this case
            self._handover_dict['message'] = msg_obj
            self._handover_cond.notifyAll()



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _urihandler module of the zhmcclient_mock package.
"""

from __future__ import absolute_import, print_function

import requests.packages.urllib3
import unittest
from mock import MagicMock

from zhmcclient_mock._hmc import FakedHmc
from zhmcclient_mock._urihandler import HTTPError, InvalidResourceError, \
    InvalidMethodError, CpcNotInDpmError, CpcInDpmError, \
    UriHandler, \
    GenericGetPropertiesHandler, GenericUpdatePropertiesHandler, \
    VersionHandler, \
    CpcsHandler, CpcHandler, CpcStartHandler, CpcStopHandler, \
    CpcExportPortNamesListHandler, \
    PartitionsHandler, PartitionHandler, PartitionStartHandler, \
    PartitionStopHandler, \
    HbasHandler, HbaHandler, \
    NicsHandler, NicHandler, \
    VirtualFunctionsHandler, VirtualFunctionHandler, \
    AdaptersHandler, AdapterHandler, \
    NetworkPortHandler, \
    StoragePortHandler, \
    VirtualSwitchesHandler, VirtualSwitchHandler, \
    LparsHandler, LparHandler, LparActivateHandler, LparDeactivateHandler, \
    LparLoadHandler, \
    ResetActProfilesHandler, ResetActProfileHandler, \
    ImageActProfilesHandler, ImageActProfileHandler, \
    LoadActProfilesHandler, LoadActProfileHandler


class HTTPErrorTests(unittest.TestCase):
    """All tests for class HTTPError."""

    def test_attributes(self):
        method = 'GET'
        uri = '/api/cpcs'
        http_status = 500
        reason = 42
        message = "fake message"

        exc = HTTPError(method, uri, http_status, reason, message)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, http_status)
        self.assertEqual(exc.reason, reason)
        self.assertEqual(exc.message, message)

    def test_response(self):
        method = 'GET'
        uri = '/api/cpcs'
        http_status = 500
        reason = 42
        message = "fake message"
        expected_response = {
            'request-method': method,
            'request-uri': uri,
            'http-status': http_status,
            'reason': reason,
            'message': message,
        }
        exc = HTTPError(method, uri, http_status, reason, message)

        response = exc.response()

        self.assertEqual(response, expected_response)


class DummyHandler1(object):
    pass


class DummyHandler2(object):
    pass


class DummyHandler3(object):
    pass


class InvalidResourceErrorTests(unittest.TestCase):
    """All tests for class InvalidResourceError."""

    def test_attributes_with_handler(self):
        method = 'GET'
        uri = '/api/cpcs'
        exp_http_status = 404
        exp_reason = 1

        exc = InvalidResourceError(method, uri, DummyHandler1)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)

    def test_attributes_no_handler(self):
        method = 'GET'
        uri = '/api/cpcs'
        exp_http_status = 404
        exp_reason = 1

        exc = InvalidResourceError(method, uri, None)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)


class InvalidMethodErrorTests(unittest.TestCase):
    """All tests for class InvalidMethodError."""

    def test_attributes_with_handler(self):
        method = 'DELETE'
        uri = '/api/cpcs'
        exp_http_status = 404
        exp_reason = 1

        exc = InvalidMethodError(method, uri, DummyHandler1)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)

    def test_attributes_no_handler(self):
        method = 'DELETE'
        uri = '/api/cpcs'
        exp_http_status = 404
        exp_reason = 1

        exc = InvalidMethodError(method, uri, None)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)


class CpcNotInDpmErrorTests(unittest.TestCase):
    """All tests for class CpcNotInDpmError."""

    def setUp(self):
        self.hmc = FakedHmc('fake-hmc', '2.13.1', '1.8')
        self.cpc1 = self.hmc.cpcs.add({'name': 'cpc1'})

    def test_attributes(self):
        method = 'GET'
        uri = '/api/cpcs/1/partitions'
        exp_http_status = 409
        exp_reason = 5

        exc = CpcNotInDpmError(method, uri, self.cpc1)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)


class CpcInDpmErrorTests(unittest.TestCase):
    """All tests for class CpcInDpmError."""

    def setUp(self):
        self.hmc = FakedHmc('fake-hmc', '2.13.1', '1.8')
        self.cpc1 = self.hmc.cpcs.add({'name': 'cpc1'})

    def test_attributes(self):
        method = 'GET'
        uri = '/api/cpcs/1/logical-partitions'
        exp_http_status = 409
        exp_reason = 4

        exc = CpcInDpmError(method, uri, self.cpc1)

        self.assertEqual(exc.method, method)
        self.assertEqual(exc.uri, uri)
        self.assertEqual(exc.http_status, exp_http_status)
        self.assertEqual(exc.reason, exp_reason)


class UriHandlerHandlerEmptyTests(unittest.TestCase):
    """All tests for UriHandler.handler() with empty URIs."""

    def setUp(self):
        self.uris = ()
        self.urihandler = UriHandler(self.uris)

    def test_uris_empty_1(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs', 'GET')

    def test_uris_empty_2(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('', 'GET')


class UriHandlerHandlerSimpleTests(unittest.TestCase):
    """All tests for UriHandler.handler() with a simple set of URIs."""

    def setUp(self):
        self.uris = (
            ('/api/cpcs', DummyHandler1),
            ('/api/cpcs/([^/]+)', DummyHandler2),
            ('/api/cpcs/([^/]+)/child', DummyHandler3),
        )
        self.urihandler = UriHandler(self.uris)

    def test_ok1(self):
        handler_class, uri_parms = self.urihandler.handler(
            '/api/cpcs', 'GET')
        self.assertEqual(handler_class, DummyHandler1)
        self.assertEqual(len(uri_parms), 0)

    def test_ok2(self):
        handler_class, uri_parms = self.urihandler.handler(
            '/api/cpcs/fake-id1', 'GET')
        self.assertEqual(handler_class, DummyHandler2)
        self.assertEqual(len(uri_parms), 1)
        self.assertEqual(uri_parms[0], 'fake-id1')

    def test_ok3(self):
        handler_class, uri_parms = self.urihandler.handler(
            '/api/cpcs/fake-id1/child', 'GET')
        self.assertEqual(handler_class, DummyHandler3)
        self.assertEqual(len(uri_parms), 1)
        self.assertEqual(uri_parms[0], 'fake-id1')

    def test_err_begin_missing(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('api/cpcs', 'GET')

    def test_err_begin_extra(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('x/api/cpcs', 'GET')

    def test_err_end_missing(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpc', 'GET')

    def test_err_end_extra(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs_x', 'GET')

    def test_err_end_slash(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs/', 'GET')

    def test_err_end2_slash(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs/fake-id1/', 'GET')

    def test_err_end2_missing(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs/fake-id1/chil', 'GET')

    def test_err_end2_extra(self):
        with self.assertRaises(InvalidResourceError):
            self.urihandler.handler('/api/cpcs/fake-id1/child_x', 'GET')


class UriHandlerMethodTests(unittest.TestCase):
    """All tests for get(), post(), delete() methods of class UriHandler."""

    def setUp(self):
        self.uris = (
            ('/api/cpcs', DummyHandler1),
            ('/api/cpcs/([^/]+)', DummyHandler2),
        )
        self.cpc1 = {
            'object-id': '1',
            'object-uri': '/api/cpcs/1',
            'name': 'cpc1',
        }
        self.cpc2 = {
            'object-id': '2',
            'object-uri': '/api/cpcs/2',
            'name': 'cpc2',
        }
        self.new_cpc = {
            'object-id': '3',
            'object-uri': '/api/cpcs/3',
            'name': 'cpc3',
        }
        self.cpcs = {
            'cpcs': [self.cpc1, self.cpc2]
        }
        DummyHandler1.get = staticmethod(MagicMock(
            return_value=self.cpcs))
        DummyHandler1.post = staticmethod(MagicMock(
            return_value=self.new_cpc))
        DummyHandler2.get = staticmethod(MagicMock(
            return_value=self.cpc1))
        DummyHandler2.delete = staticmethod(MagicMock(
            return_value=None))
        self.urihandler = UriHandler(self.uris)
        self.hmc = 'fake-hmc-object'

    def tearDown(self):
        delattr(DummyHandler1, 'get')
        delattr(DummyHandler1, 'post')
        delattr(DummyHandler2, 'get')
        delattr(DummyHandler2, 'delete')

    def test_get_cpcs(self):

        # the function to be tested
        result = self.urihandler.get(self.hmc, '/api/cpcs', True)

        self.assertEqual(result, self.cpcs)

        DummyHandler1.get.assert_called_with(self.hmc, '/api/cpcs', tuple(),
                                             True)
        self.assertEqual(DummyHandler1.post.called, 0)
        self.assertEqual(DummyHandler2.get.called, 0)
        self.assertEqual(DummyHandler2.delete.called, 0)

    def test_get_cpc1(self):

        # the function to be tested
        result = self.urihandler.get(self.hmc, '/api/cpcs/1', True)

        self.assertEqual(result, self.cpc1)

        self.assertEqual(DummyHandler1.get.called, 0)
        self.assertEqual(DummyHandler1.post.called, 0)
        DummyHandler2.get.assert_called_with(self.hmc, '/api/cpcs/1',
                                             tuple('1'), True)
        self.assertEqual(DummyHandler2.delete.called, 0)

    def test_post_cpcs(self):

        # the function to be tested
        result = self.urihandler.post(self.hmc, '/api/cpcs', {}, True, True)

        self.assertEqual(result, self.new_cpc)

        self.assertEqual(DummyHandler1.get.called, 0)
        DummyHandler1.post.assert_called_with(self.hmc, '/api/cpcs', tuple(),
                                              {}, True, True)
        self.assertEqual(DummyHandler2.get.called, 0)
        self.assertEqual(DummyHandler2.delete.called, 0)

    def test_delete_cpc2(self):

        # the function to be tested
        self.urihandler.delete(self.hmc, '/api/cpcs/2', True)

        self.assertEqual(DummyHandler1.get.called, 0)
        self.assertEqual(DummyHandler1.post.called, 0)
        self.assertEqual(DummyHandler2.get.called, 0)
        DummyHandler2.delete.assert_called_with(self.hmc, '/api/cpcs/2',
                                                tuple('2'), True)


def standard_test_hmc():
    """
    Return a FakedHmc object that is prepared with a few standard resources
    for testing.
    """
    hmc_resources = {
        'cpcs': [
            {
                'properties': {
                    'object-id': '1',
                    'name': 'cpc_1',
                    'dpm-enabled': False,
                    'description': 'CPC #1 (classic mode)',
                    'status': 'operating',
                },
                'lpars': [
                    {
                        'properties': {
                            'object-id': '1',
                            'name': 'lpar_1',
                            'description': 'LPAR #1 in CPC #1',
                            'status': 'not-activated',
                        },
                    },
                ],
                'reset_activation_profiles': [
                    {
                        'properties': {
                            'element-id': '1',
                            'name': 'rap_1',
                            'description': 'Reset profile #1 in CPC #1',
                        },
                    },
                ],
                'image_activation_profiles': [
                    {
                        'properties': {
                            'element-id': '1',
                            'name': 'iap_1',
                            'description': 'Image profile #1 in CPC #1',
                        },
                    },
                ],
                'load_activation_profiles': [
                    {
                        'properties': {
                            'element-id': '1',
                            'name': 'lap_1',
                            'description': 'Load profile #1 in CPC #1',
                        },
                    },
                ],
            },
            {
                'properties': {
                    'object-id': '2',
                    'name': 'cpc_2',
                    'dpm-enabled': True,
                    'description': 'CPC #2 (DPM mode)',
                    'status': 'active',
                },
                'partitions': [
                    {
                        'properties': {
                            'object-id': '1',
                            'name': 'partition_1',
                            'description': 'Partition #1 in CPC #2',
                            'status': 'stopped',
                            'hba-uris': [],   # updated automatically
                            'nic-uris': [],   # updated automatically
                            'virtual-function-uris': [],   # updated autom.
                        },
                        'hbas': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'hba_1',
                                    'description': 'HBA #1 in Partition #1',
                                    'adapter-port-uri':
                                        '/api/adapters/2/storage-ports/1',
                                    'wwpn': 'CFFEAFFE00008001',
                                    'device-number': '1001',
                                },
                            },
                        ],
                        'nics': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'nic_1',
                                    'description': 'NIC #1 in Partition #1',
                                    'network-adapter-port-uri':
                                        '/api/adapters/3/network-ports/1',
                                    'device-number': '2001',
                                },
                            },
                        ],
                        'virtual_functions': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'vf_1',
                                    'description': 'VF #1 in Partition #1',
                                    'device-number': '3001',
                                },
                            },
                        ],
                    },
                ],
                'adapters': [
                    {
                        'properties': {
                            'object-id': '1',
                            'name': 'osa_1',
                            'description': 'OSA #1 in CPC #2',
                            'adapter-family': 'osa',
                            'network-port-uris': [],   # updated automatically
                            'status': 'active',
                        },
                        'ports': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'osa_1_port_1',
                                    'description': 'Port #1 of OSA #1',
                                },
                            },
                        ],
                    },
                    {
                        'properties': {
                            'object-id': '2',
                            'name': 'fcp_2',
                            'description': 'FCP #2 in CPC #2',
                            'adapter-family': 'ficon',
                            'storage-port-uris': [],   # updated automatically
                        },
                        'ports': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'fcp_2_port_1',
                                    'description': 'Port #1 of FCP #2',
                                },
                            },
                        ],
                    },
                    {
                        'properties': {
                            'object-id': '3',
                            'name': 'roce_3',
                            'description': 'ROCE #3 in CPC #2',
                            'adapter-family': 'roce',
                            'network-port-uris': [],   # updated automatically
                        },
                        'ports': [
                            {
                                'properties': {
                                    'element-id': '1',
                                    'name': 'roce_3_port_1',
                                    'description': 'Port #1 of ROCE #3',
                                },
                            },
                        ],
                    },
                ],
                'virtual_switches': [
                    {
                        'properties': {
                            'object-id': '1',
                            'name': 'vswitch_osa_1',
                            'description': 'Vswitch for OSA #1 in CPC #2',
                        },
                    },
                ],
            },
        ],
    }
    hmc = FakedHmc('fake-hmc', '2.13.1', '1.8')
    hmc.add_resources(hmc_resources)
    return hmc, hmc_resources


class GenericGetPropertiesHandlerTests(unittest.TestCase):
    """All tests for class GenericGetPropertiesHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)', GenericGetPropertiesHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_get(self):

        # the function to be tested:
        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)

        exp_cpc1 = {
            'object-id': '1',
            'object-uri': '/api/cpcs/1',
            'name': 'cpc_1',
            'dpm-enabled': False,
            'is-ensemble-member': False,
            'description': 'CPC #1 (classic mode)',
            'status': 'operating',
        }
        self.assertEqual(cpc1, exp_cpc1)


class _GenericGetUpdatePropertiesHandler(GenericGetPropertiesHandler,
                                         GenericUpdatePropertiesHandler):
    pass


class GenericUpdatePropertiesHandlerTests(unittest.TestCase):
    """All tests for class GenericUpdatePropertiesHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)', _GenericGetUpdatePropertiesHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_update_verify(self):
        update_cpc1 = {
            'description': 'CPC #1 (updated)',
        }

        # the function to be tested:
        resp = self.urihandler.post(self.hmc, '/api/cpcs/1', update_cpc1, True,
                                    True)

        self.assertEqual(resp, None)
        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['description'], 'CPC #1 (updated)')


class VersionHandlerTests(unittest.TestCase):
    """All tests for class VersionHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/version', VersionHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_get_version(self):

        # the function to be tested:
        resp = self.urihandler.get(self.hmc, '/api/version', True)

        api_major, api_minor = self.hmc.api_version.split('.')
        exp_resp = {
            'hmc-name': self.hmc.hmc_name,
            'hmc-version': self.hmc.hmc_version,
            'api-major-version': int(api_major),
            'api-minor-version': int(api_minor),
        }
        self.assertEqual(resp, exp_resp)


class CpcHandlersTests(unittest.TestCase):
    """All tests for classes CpcsHandler and CpcHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs', CpcsHandler),
            ('/api/cpcs/([^/]+)', CpcHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        cpcs = self.urihandler.get(self.hmc, '/api/cpcs', True)

        exp_cpcs = {  # properties reduced to those returned by List
            'cpcs': [
                {
                    'object-uri': '/api/cpcs/1',
                    'name': 'cpc_1',
                    'status': 'operating',
                },
                {
                    'object-uri': '/api/cpcs/2',
                    'name': 'cpc_2',
                    'status': 'active',
                },
            ]
        }
        self.assertEqual(cpcs, exp_cpcs)

    def test_get(self):

        # the function to be tested:
        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)

        exp_cpc1 = {
            'object-id': '1',
            'object-uri': '/api/cpcs/1',
            'name': 'cpc_1',
            'dpm-enabled': False,
            'is-ensemble-member': False,
            'description': 'CPC #1 (classic mode)',
            'status': 'operating',
        }
        self.assertEqual(cpc1, exp_cpc1)

    def test_update_verify(self):
        update_cpc1 = {
            'description': 'updated cpc #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/cpcs/1',
                             update_cpc1, True, True)

        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['description'], 'updated cpc #1')


class CpcStartStopHandlerTests(unittest.TestCase):
    """All tests for classes CpcStartHandler and CpcStopHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)', CpcHandler),
            ('/api/cpcs/([^/]+)/operations/start', CpcStartHandler),
            ('/api/cpcs/([^/]+)/operations/stop', CpcStopHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_stop_classic(self):
        # CPC1 is in classic mode
        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['status'], 'operating')

        # the function to be tested:
        with self.assertRaises(CpcNotInDpmError):
            self.urihandler.post(self.hmc, '/api/cpcs/1/operations/stop',
                                 None, True, True)

        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['status'], 'operating')

    def test_start_classic(self):
        # CPC1 is in classic mode
        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['status'], 'operating')

        # the function to be tested:
        with self.assertRaises(CpcNotInDpmError):
            self.urihandler.post(self.hmc, '/api/cpcs/1/operations/start',
                                 None, True, True)

        cpc1 = self.urihandler.get(self.hmc, '/api/cpcs/1', True)
        self.assertEqual(cpc1['status'], 'operating')

    def test_stop_start_dpm(self):
        # CPC2 is in DPM mode
        cpc2 = self.urihandler.get(self.hmc, '/api/cpcs/2', True)
        self.assertEqual(cpc2['status'], 'active')

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/cpcs/2/operations/stop',
                             None, True, True)

        cpc2 = self.urihandler.get(self.hmc, '/api/cpcs/2', True)
        self.assertEqual(cpc2['status'], 'not-operating')

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/cpcs/2/operations/start',
                             None, True, True)

        cpc2 = self.urihandler.get(self.hmc, '/api/cpcs/2', True)
        self.assertEqual(cpc2['status'], 'active')


class CpcExportPortNamesListHandlerTests(unittest.TestCase):
    """All tests for class CpcExportPortNamesListHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs', CpcsHandler),
            ('/api/cpcs/([^/]+)', CpcHandler),
            ('/api/cpcs/([^/]+)/operations/export-port-names-list',
             CpcExportPortNamesListHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_invoke_err_no_input(self):

        # the function to be tested:
        with self.assertRaises(HTTPError):
            self.urihandler.post(
                self.hmc, '/api/cpcs/2/operations/export-port-names-list',
                None, True, True)

    def test_invoke_ok(self):
        operation_body = {
            'partitions': [
                '/api/partitions/1',
            ]
        }
        exp_wwpn_list = [
            'partition_1,2,1001,CFFEAFFE00008001',
        ]

        # the function to be tested:
        resp = self.urihandler.post(
            self.hmc, '/api/cpcs/2/operations/export-port-names-list',
            operation_body, True, True)

        self.assertEqual(len(resp), 1)
        self.assertIn('wwpn-list', resp)
        wwpn_list = resp['wwpn-list']
        self.assertEqual(wwpn_list, exp_wwpn_list)


class AdapterHandlersTests(unittest.TestCase):
    """All tests for classes AdaptersHandler and AdapterHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/adapters', AdaptersHandler),
            ('/api/adapters/([^/]+)', AdapterHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        adapters = self.urihandler.get(self.hmc, '/api/cpcs/2/adapters', True)

        exp_adapters = {  # properties reduced to those returned by List
            'adapters': [
                {
                    'object-uri': '/api/adapters/1',
                    'name': 'osa_1',
                    'status': 'active',
                },
                {
                    'object-uri': '/api/adapters/2',
                    'name': 'fcp_2',
                    'status': 'active',
                },
                {
                    'object-uri': '/api/adapters/3',
                    'name': 'roce_3',
                    'status': 'active',
                },
            ]
        }
        self.assertEqual(adapters, exp_adapters)

    def test_get(self):

        # the function to be tested:
        adapter1 = self.urihandler.get(self.hmc, '/api/adapters/1', True)

        exp_adapter1 = {
            'object-id': '1',
            'object-uri': '/api/adapters/1',
            'name': 'osa_1',
            'description': 'OSA #1 in CPC #2',
            'status': 'active',
            'adapter-family': 'osa',
            'network-port-uris': ['/api/adapters/1/network-ports/1'],
        }
        self.assertEqual(adapter1, exp_adapter1)

    def test_update_verify(self):
        update_adapter1 = {
            'description': 'updated adapter #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/adapters/1',
                             update_adapter1, True, True)

        adapter1 = self.urihandler.get(self.hmc, '/api/adapters/1', True)
        self.assertEqual(adapter1['description'], 'updated adapter #1')


class NetworkPortHandlersTests(unittest.TestCase):
    """All tests for class NetworkPortHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/adapters/([^/]+)/network-ports/([^/]+)',
             NetworkPortHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_get(self):

        # the function to be tested:
        port1 = self.urihandler.get(self.hmc,
                                    '/api/adapters/1/network-ports/1', True)

        exp_port1 = {
            'element-id': '1',
            'element-uri': '/api/adapters/1/network-ports/1',
            'name': 'osa_1_port_1',
            'description': 'Port #1 of OSA #1',
        }
        self.assertEqual(port1, exp_port1)

    def test_update_verify(self):
        update_port1 = {
            'description': 'updated port #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/adapters/1/network-ports/1',
                             update_port1, True, True)

        port1 = self.urihandler.get(self.hmc,
                                    '/api/adapters/1/network-ports/1', True)
        self.assertEqual(port1['description'], 'updated port #1')


class StoragePortHandlersTests(unittest.TestCase):
    """All tests for class StoragePortHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/adapters/([^/]+)/storage-ports/([^/]+)',
             StoragePortHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_get(self):

        # the function to be tested:
        port1 = self.urihandler.get(self.hmc,
                                    '/api/adapters/2/storage-ports/1', True)

        exp_port1 = {
            'element-id': '1',
            'element-uri': '/api/adapters/2/storage-ports/1',
            'name': 'fcp_2_port_1',
            'description': 'Port #1 of FCP #2',
        }
        self.assertEqual(port1, exp_port1)

    def test_update_verify(self):
        update_port1 = {
            'description': 'updated port #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/adapters/2/storage-ports/1',
                             update_port1, True, True)

        port1 = self.urihandler.get(self.hmc,
                                    '/api/adapters/2/storage-ports/1', True)
        self.assertEqual(port1['description'], 'updated port #1')


class PartitionHandlersTests(unittest.TestCase):
    """All tests for classes PartitionsHandler and PartitionHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/partitions', PartitionsHandler),
            ('/api/partitions/([^/]+)', PartitionHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        partitions = self.urihandler.get(self.hmc, '/api/cpcs/2/partitions',
                                         True)

        exp_partitions = {  # properties reduced to those returned by List
            'partitions': [
                {
                    'object-uri': '/api/partitions/1',
                    'name': 'partition_1',
                    'status': 'stopped',
                },
            ]
        }
        self.assertEqual(partitions, exp_partitions)

    def test_get(self):

        # the function to be tested:
        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)

        exp_partition1 = {
            'object-id': '1',
            'object-uri': '/api/partitions/1',
            'name': 'partition_1',
            'description': 'Partition #1 in CPC #2',
            'status': 'stopped',
            'hba-uris': ['/api/partitions/1/hbas/1'],
            'nic-uris': ['/api/partitions/1/nics/1'],
            'virtual-function-uris': ['/api/partitions/1/virtual-functions/1'],
        }
        self.assertEqual(partition1, exp_partition1)

    def test_create_verify(self):
        new_partition2 = {
            'object-id': '2',
            'name': 'partition_2',
        }

        # the function to be tested:
        resp = self.urihandler.post(self.hmc, '/api/cpcs/2/partitions',
                                    new_partition2, True, True)

        self.assertEqual(len(resp), 1)
        self.assertIn('object-uri', resp)
        new_partition2_uri = resp['object-uri']
        self.assertEqual(new_partition2_uri, '/api/partitions/2')

        exp_partition2 = {
            'object-id': '2',
            'object-uri': '/api/partitions/2',
            'name': 'partition_2',
            'status': 'stopped',
            'hba-uris': [],
            'nic-uris': [],
            'virtual-function-uris': [],
        }

        # the function to be tested:
        partition2 = self.urihandler.get(self.hmc, '/api/partitions/2', True)

        self.assertEqual(partition2, exp_partition2)

    def test_update_verify(self):
        update_partition1 = {
            'description': 'updated partition #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1',
                             update_partition1, True, True)

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        self.assertEqual(partition1['description'], 'updated partition #1')

    def test_delete_verify(self):

        self.urihandler.get(self.hmc, '/api/partitions/1', True)

        # the function to be tested:
        self.urihandler.delete(self.hmc, '/api/partitions/1', True)

        with self.assertRaises(InvalidResourceError):
            self.urihandler.get(self.hmc, '/api/partitions/1', True)


class PartitionStartStopHandlerTests(unittest.TestCase):
    """All tests for classes PartitionStartHandler and PartitionStopHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/partitions/([^/]+)', PartitionHandler),
            ('/api/partitions/([^/]+)/operations/start',
             PartitionStartHandler),
            ('/api/partitions/([^/]+)/operations/stop', PartitionStopHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_start_stop(self):
        # CPC2 is in DPM mode
        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        self.assertEqual(partition1['status'], 'stopped')

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1/operations/start',
                             None, True, True)

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        self.assertEqual(partition1['status'], 'active')

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1/operations/stop',
                             None, True, True)

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        self.assertEqual(partition1['status'], 'stopped')


class HbaHandlerTests(unittest.TestCase):
    """All tests for classes HbasHandler and HbaHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/partitions/([^/]+)', PartitionHandler),
            ('/api/partitions/([^/]+)/hbas', HbasHandler),
            ('/api/partitions/([^/]+)/hbas/([^/]+)', HbaHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)

        hba_uris = partition1.get('hba-uris', [])

        exp_hba_uris = [
            '/api/partitions/1/hbas/1',
        ]
        self.assertEqual(hba_uris, exp_hba_uris)

    def test_get(self):

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        hba1_uri = partition1.get('hba-uris', [])[0]

        # the function to be tested:
        hba1 = self.urihandler.get(self.hmc, hba1_uri, True)

        exp_hba1 = {
            'element-id': '1',
            'element-uri': '/api/partitions/1/hbas/1',
            'name': 'hba_1',
            'description': 'HBA #1 in Partition #1',
            'adapter-port-uri': '/api/adapters/2/storage-ports/1',
            'wwpn': 'CFFEAFFE00008001',
            'device-number': '1001',
        }
        self.assertEqual(hba1, exp_hba1)

    def test_create_verify(self):
        new_hba2 = {
            'element-id': '2',
            'name': 'hba_2',
            'adapter-port-uri': '/api/adapters/2/storage-ports/1',
        }

        # the function to be tested:
        resp = self.urihandler.post(self.hmc, '/api/partitions/1/hbas',
                                    new_hba2, True, True)

        self.assertEqual(len(resp), 1)
        self.assertIn('element-uri', resp)
        new_hba2_uri = resp['element-uri']
        self.assertEqual(new_hba2_uri, '/api/partitions/1/hbas/2')

        # the function to be tested:
        hba2 = self.urihandler.get(self.hmc, '/api/partitions/1/hbas/2', True)

        exp_hba2 = {
            'element-id': '2',
            'element-uri': '/api/partitions/1/hbas/2',
            'name': 'hba_2',
            'adapter-port-uri': '/api/adapters/2/storage-ports/1',
            'device-number': hba2['device-number'],  # auto-generated
            'wwpn': hba2['wwpn'],  # auto-generated
        }

        self.assertEqual(hba2, exp_hba2)

    def test_update_verify(self):
        update_hba1 = {
            'description': 'updated hba #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1/hbas/1',
                             update_hba1, True, True)

        hba1 = self.urihandler.get(self.hmc, '/api/partitions/1/hbas/1', True)
        self.assertEqual(hba1['description'], 'updated hba #1')

    def test_delete_verify(self):

        self.urihandler.get(self.hmc, '/api/partitions/1/hbas/1', True)

        # the function to be tested:
        self.urihandler.delete(self.hmc, '/api/partitions/1/hbas/1', True)

        with self.assertRaises(InvalidResourceError):
            self.urihandler.get(self.hmc, '/api/partitions/1/hbas/1', True)


class NicHandlerTests(unittest.TestCase):
    """All tests for classes NicsHandler and NicHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/partitions/([^/]+)', PartitionHandler),
            ('/api/partitions/([^/]+)/nics', NicsHandler),
            ('/api/partitions/([^/]+)/nics/([^/]+)', NicHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)

        nic_uris = partition1.get('nic-uris', [])

        exp_nic_uris = [
            '/api/partitions/1/nics/1',
        ]
        self.assertEqual(nic_uris, exp_nic_uris)

    def test_get(self):

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        nic1_uri = partition1.get('nic-uris', [])[0]

        # the function to be tested:
        nic1 = self.urihandler.get(self.hmc, nic1_uri, True)

        exp_nic1 = {
            'element-id': '1',
            'element-uri': '/api/partitions/1/nics/1',
            'name': 'nic_1',
            'description': 'NIC #1 in Partition #1',
            'network-adapter-port-uri': '/api/adapters/3/network-ports/1',
            'device-number': '2001',
        }
        self.assertEqual(nic1, exp_nic1)

    def test_create_verify(self):
        new_nic2 = {
            'element-id': '2',
            'name': 'nic_2',
            'network-adapter-port-uri': '/api/adapters/3/network-ports/1',
        }

        # the function to be tested:
        resp = self.urihandler.post(self.hmc, '/api/partitions/1/nics',
                                    new_nic2, True, True)

        self.assertEqual(len(resp), 1)
        self.assertIn('element-uri', resp)
        new_nic2_uri = resp['element-uri']
        self.assertEqual(new_nic2_uri, '/api/partitions/1/nics/2')

        # the function to be tested:
        nic2 = self.urihandler.get(self.hmc, '/api/partitions/1/nics/2', True)

        exp_nic2 = {
            'element-id': '2',
            'element-uri': '/api/partitions/1/nics/2',
            'name': 'nic_2',
            'network-adapter-port-uri': '/api/adapters/3/network-ports/1',
            'device-number': nic2['device-number'],  # auto-generated
        }

        self.assertEqual(nic2, exp_nic2)

    def test_update_verify(self):
        update_nic1 = {
            'description': 'updated nic #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1/nics/1',
                             update_nic1, True, True)

        nic1 = self.urihandler.get(self.hmc, '/api/partitions/1/nics/1', True)
        self.assertEqual(nic1['description'], 'updated nic #1')

    def test_delete_verify(self):

        self.urihandler.get(self.hmc, '/api/partitions/1/nics/1', True)

        # the function to be tested:
        self.urihandler.delete(self.hmc, '/api/partitions/1/nics/1', True)

        with self.assertRaises(InvalidResourceError):
            self.urihandler.get(self.hmc, '/api/partitions/1/nics/1', True)


class VirtualFunctionHandlerTests(unittest.TestCase):
    """All tests for classes VirtualFunctionsHandler and
    VirtualFunctionHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/partitions/([^/]+)', PartitionHandler),
            ('/api/partitions/([^/]+)/virtual-functions',
             VirtualFunctionsHandler),
            ('/api/partitions/([^/]+)/virtual-functions/([^/]+)',
             VirtualFunctionHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)

        vf_uris = partition1.get('virtual-function-uris', [])

        exp_vf_uris = [
            '/api/partitions/1/virtual-functions/1',
        ]
        self.assertEqual(vf_uris, exp_vf_uris)

    def test_get(self):

        partition1 = self.urihandler.get(self.hmc, '/api/partitions/1', True)
        vf1_uri = partition1.get('virtual-function-uris', [])[0]

        # the function to be tested:
        vf1 = self.urihandler.get(self.hmc, vf1_uri, True)

        exp_vf1 = {
            'element-id': '1',
            'element-uri': '/api/partitions/1/virtual-functions/1',
            'name': 'vf_1',
            'description': 'VF #1 in Partition #1',
            'device-number': '3001',
        }
        self.assertEqual(vf1, exp_vf1)

    def test_create_verify(self):
        new_vf2 = {
            'element-id': '2',
            'name': 'vf_2',
        }

        # the function to be tested:
        resp = self.urihandler.post(self.hmc,
                                    '/api/partitions/1/virtual-functions',
                                    new_vf2, True, True)

        self.assertEqual(len(resp), 1)
        self.assertIn('element-uri', resp)
        new_vf2_uri = resp['element-uri']
        self.assertEqual(new_vf2_uri, '/api/partitions/1/virtual-functions/2')

        # the function to be tested:
        vf2 = self.urihandler.get(self.hmc,
                                  '/api/partitions/1/virtual-functions/2',
                                  True)

        exp_vf2 = {
            'element-id': '2',
            'element-uri': '/api/partitions/1/virtual-functions/2',
            'name': 'vf_2',
            'device-number': vf2['device-number'],  # auto-generated
        }

        self.assertEqual(vf2, exp_vf2)

    def test_update_verify(self):
        update_vf1 = {
            'description': 'updated vf #1',
        }

        # the function to be tested:
        self.urihandler.post(self.hmc, '/api/partitions/1/virtual-functions/1',
                             update_vf1, True, True)

        vf1 = self.urihandler.get(self.hmc,
                                  '/api/partitions/1/virtual-functions/1',
                                  True)
        self.assertEqual(vf1['description'], 'updated vf #1')

    def test_delete_verify(self):

        self.urihandler.get(self.hmc, '/api/partitions/1/virtual-functions/1',
                            True)

        # the function to be tested:
        self.urihandler.delete(self.hmc,
                               '/api/partitions/1/virtual-functions/1', True)

        with self.assertRaises(InvalidResourceError):
            self.urihandler.get(self.hmc,
                                '/api/partitions/1/virtual-functions/1', True)


class VirtualSwitchHandlersTests(unittest.TestCase):
    """All tests for classes VirtualSwitchesHandler and
    VirtualSwitchHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/virtual-switches', VirtualSwitchesHandler),
            ('/api/virtual-switches/([^/]+)', VirtualSwitchHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        vswitches = self.urihandler.get(self.hmc,
                                        '/api/cpcs/2/virtual-switches', True)

        exp_vswitches = {  # properties reduced to those returned by List
            'virtual-switches': [
                {
                    'object-uri': '/api/virtual-switches/1',
                    'name': 'vswitch_osa_1',
                    # status not set in resource -> not in response
                },
            ]
        }
        self.assertEqual(vswitches, exp_vswitches)

    def test_get(self):

        # the function to be tested:
        vswitch1 = self.urihandler.get(self.hmc, '/api/virtual-switches/1',
                                       True)

        exp_vswitch1 = {
            'object-id': '1',
            'object-uri': '/api/virtual-switches/1',
            'name': 'vswitch_osa_1',
            'description': 'Vswitch for OSA #1 in CPC #2',
        }
        self.assertEqual(vswitch1, exp_vswitch1)


class LparHandlersTests(unittest.TestCase):
    """All tests for classes LparsHandler and LparHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/logical-partitions', LparsHandler),
            ('/api/logical-partitions/([^/]+)', LparHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        lpars = self.urihandler.get(self.hmc,
                                    '/api/cpcs/1/logical-partitions', True)

        exp_lpars = {  # properties reduced to those returned by List
            'logical-partitions': [
                {
                    'object-uri': '/api/logical-partitions/1',
                    'name': 'lpar_1',
                    'status': 'not-activated',
                },
            ]
        }
        self.assertEqual(lpars, exp_lpars)

    def test_get(self):

        # the function to be tested:
        lpar1 = self.urihandler.get(self.hmc, '/api/logical-partitions/1',
                                    True)

        exp_lpar1 = {
            'object-id': '1',
            'object-uri': '/api/logical-partitions/1',
            'name': 'lpar_1',
            'status': 'not-activated',
            'description': 'LPAR #1 in CPC #1',
        }
        self.assertEqual(lpar1, exp_lpar1)


class LparActLoadDeactHandlerTests(unittest.TestCase):
    """All tests for classes LparActivateHandler, LparLoadHandler, and
    LparDeactivateHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/logical-partitions/([^/]+)',
             LparHandler),
            ('/api/logical-partitions/([^/]+)/operations/activate',
             LparActivateHandler),
            ('/api/logical-partitions/([^/]+)/operations/deactivate',
             LparDeactivateHandler),
            ('/api/logical-partitions/([^/]+)/operations/load',
             LparLoadHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_start_stop(self):
        # CPC1 is in classic mode
        lpar1 = self.urihandler.get(self.hmc, '/api/logical-partitions/1',
                                    True)
        self.assertEqual(lpar1['status'], 'not-activated')

        # the function to be tested:
        self.urihandler.post(self.hmc,
                             '/api/logical-partitions/1/operations/activate',
                             None, True, True)

        lpar1 = self.urihandler.get(self.hmc, '/api/logical-partitions/1',
                                    True)
        self.assertEqual(lpar1['status'], 'not-operating')

        # the function to be tested:
        self.urihandler.post(self.hmc,
                             '/api/logical-partitions/1/operations/load',
                             None, True, True)

        lpar1 = self.urihandler.get(self.hmc, '/api/logical-partitions/1',
                                    True)
        self.assertEqual(lpar1['status'], 'operating')

        # the function to be tested:
        self.urihandler.post(self.hmc,
                             '/api/logical-partitions/1/operations/deactivate',
                             None, True, True)

        lpar1 = self.urihandler.get(self.hmc, '/api/logical-partitions/1',
                                    True)
        self.assertEqual(lpar1['status'], 'not-activated')


class ResetActProfileHandlersTests(unittest.TestCase):
    """All tests for classes ResetActProfilesHandler and
    ResetActProfileHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/reset-activation-profiles',
             ResetActProfilesHandler),
            ('/api/cpcs/([^/]+)/reset-activation-profiles/([^/]+)',
             ResetActProfileHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        raps = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/reset-activation-profiles',
                                   True)

        exp_raps = {  # properties reduced to those returned by List
            'reset-activation-profiles': [
                {
                    'element-uri': '/api/cpcs/1/reset-activation-profiles/1',
                    'name': 'rap_1',
                },
            ]
        }
        self.assertEqual(raps, exp_raps)

    def test_get(self):

        # the function to be tested:
        rap1 = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/reset-activation-profiles/1',
                                   True)

        exp_rap1 = {
            'element-id': '1',
            'element-uri': '/api/cpcs/1/reset-activation-profiles/1',
            'name': 'rap_1',
            'description': 'Reset profile #1 in CPC #1',
        }
        self.assertEqual(rap1, exp_rap1)


class ImageActProfileHandlersTests(unittest.TestCase):
    """All tests for classes ImageActProfilesHandler and
    ImageActProfileHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/image-activation-profiles/([^/]+)',
             ImageActProfileHandler),
            ('/api/cpcs/([^/]+)/image-activation-profiles',
             ImageActProfilesHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        iaps = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/image-activation-profiles',
                                   True)

        exp_iaps = {  # properties reduced to those returned by List
            'image-activation-profiles': [
                {
                    'element-uri': '/api/cpcs/1/image-activation-profiles/1',
                    'name': 'iap_1',
                },
            ]
        }
        self.assertEqual(iaps, exp_iaps)

    def test_get(self):

        # the function to be tested:
        iap1 = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/image-activation-profiles/1',
                                   True)

        exp_iap1 = {
            'element-id': '1',
            'element-uri': '/api/cpcs/1/image-activation-profiles/1',
            'name': 'iap_1',
            'description': 'Image profile #1 in CPC #1',
        }
        self.assertEqual(iap1, exp_iap1)


class LoadActProfileHandlersTests(unittest.TestCase):
    """All tests for classes LoadActProfilesHandler and
    LoadActProfileHandler."""

    def setUp(self):
        self.hmc, self.hmc_resources = standard_test_hmc()
        self.uris = (
            ('/api/cpcs/([^/]+)/load-activation-profiles/([^/]+)',
             LoadActProfileHandler),
            ('/api/cpcs/([^/]+)/load-activation-profiles',
             LoadActProfilesHandler),
        )
        self.urihandler = UriHandler(self.uris)

    def test_list(self):

        # the function to be tested:
        laps = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/load-activation-profiles',
                                   True)

        exp_laps = {  # properties reduced to those returned by List
            'load-activation-profiles': [
                {
                    'element-uri': '/api/cpcs/1/load-activation-profiles/1',
                    'name': 'lap_1',
                },
            ]
        }
        self.assertEqual(laps, exp_laps)

    def test_get(self):

        # the function to be tested:
        lap1 = self.urihandler.get(self.hmc,
                                   '/api/cpcs/1/load-activation-profiles/1',
                                   True)

        exp_lap1 = {
            'element-id': '1',
            'element-uri': '/api/cpcs/1/load-activation-profiles/1',
            'name': 'lap_1',
            'description': 'Load profile #1 in CPC #1',
        }
        self.assertEqual(lap1, exp_lap1)


if __name__ == '__main__':
    requests.packages.urllib3.disable_warnings()
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Base definitions for resource manager classes.

Resource manager classes exist for each resource type and are helper classes
that provide functionality common for the resource type.

Resource manager objects are not necessarily singleton objects, because they
have a scope of a certain set of resource objects. For example, the resource
manager object for LPARs exists once for each CPC managed by the HMC, and the
resource object scope of each LPAR manager object is the set of LPARs in that
CPC.
"""

from __future__ import absolute_import

from requests.utils import quote

from ._logging import get_logger, logged_api_call
from ._exceptions import NotFound, NoUniqueMatch

__all__ = ['BaseManager']

LOG = get_logger(__name__)


class BaseManager(object):
    """
    Abstract base class for manager classes (e.g.
    :class:`~zhmcclient.CpcManager`).

    It defines the interface for the derived manager classes, and implements
    methods that have a common implementation for the derived manager classes.

    Objects of derived manager classes should not be created by users of this
    package by simply instantiating them. Instead, such objects are created by
    this package as instance variables of :class:`~zhmcclient.Client` and
    other resource objects, e.g. :attr:`~zhmcclient.Client.cpcs`. For this
    reason, the `__init__()`  method of this class and of its derived manager
    classes are considered internal interfaces and their parameters are not
    documented and may change incompatibly.
    """

    def __init__(self, resource_class, parent, uri_prop, name_prop,
                 query_props):
        # This method intentionally has no docstring, because it is internal.
        #
        # Parameters:
        #   resource_class (class):
        #     Python class for the resources of this manager.
        #     Must not be `None`.
        #   parent (subclass of :class:`~zhmcclient.BaseResource`):
        #     Parent resource defining the scope for this manager.
        #     `None`, if the manager has no parent, i.e. when it manages
        #     top-level resources (e.g. CPC).
        #   uri_prop (string):
        #     Name of the resource property that is the canonical URI path of
        #     the resource (e.g. 'object-uri' or 'element-uri').
        #     Must not be `None`.
        #   name_prop (string):
        #     Name of the resource property that is the name of the resource
        #     (e.g. 'name').
        #     Must not be `None`.
        #   query_props (iterable of strings):
        #     List of names of resource properties that are supported as filter
        #     query parameters in HMC list operations for this type of resource
        #     (i.e. for server-side filtering).
        #     Must not be `None`.
        #     If the support for a resource property changes within the set of
        #     HMC versions that support this type of resource, this list must
        #     represent the version of the HMC this session is connected to.

        # We want to surface precondition violations as early as possible,
        # so we test those that are not surfaced through the init code:
        assert resource_class is not None
        assert uri_prop is not None
        assert name_prop is not None
        assert query_props is not None

        self._resource_class = resource_class
        self._parent = parent
        self._uri_prop = uri_prop
        self._name_prop = name_prop
        self._query_props = query_props

        self._uris = {}

        # Note: Managers of top-level resources must update the following
        # instance variables in their init:
        self._session = parent.manager.session if parent else None

    def _get_uri(self, name):
        """
        Look up a resource of this manager by name, using and possibly
        refreshing the cached name-to-URI mapping.
        """
        try:
            return self._uris[name]
        except KeyError:
            res_list = self.list()
            for res in res_list:
                self._uris[res.name] = res.uri
            try:
                return self._uris[name]
            except KeyError:
                raise NotFound

    def _divide_filter_args(self, filter_args):
        """
        Divide the filter arguments into filter query parameters for filtering
        on the server side, and the remaining client-side filters.

        Parameters:

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          : tuple (query_parms_str, client_filter_args)
        """
        query_parms = []  # query parameter strings
        client_filter_args = {}

        if filter_args is not None:
            for prop_name in filter_args:
                prop_match = filter_args[prop_name]
                if prop_name in self._query_props:
                    self._append_query_parms(query_parms, prop_name,
                                             prop_match)
                else:
                    client_filter_args[prop_name] = prop_match
        query_parms_str = '&'.join(query_parms)
        if query_parms_str:
            query_parms_str = '?{}'.format(query_parms_str)

        return query_parms_str, client_filter_args

    def _append_query_parms(self, query_parms, prop_name, prop_match):
        if isinstance(prop_match, (list, tuple)):
            for pm in prop_match:
                self._append_query_parms(query_parms, prop_name, pm)
        else:
            # Just in case, we also escape the property name
            parm_name = quote(prop_name, safe='')
            parm_value = quote(str(prop_match), safe='')
            qp = '{}={}'.format(parm_name, parm_value)
            query_parms.append(qp)

    def _matches_filters(self, obj, filter_args):
        """
        Return a boolean indicating whether a resource object matches a set
        of filter arguments.
        This is used for client-side filtering.

        Depending on the properties specified in the filter arguments, this
        method retrieves the resource properties from the HMC.

        Parameters:

          obj (BaseResource):
            Resource object.

          filter_args (dict):
            Filter arguments. For details, see :ref:`Filtering`.
            `None` causes the resource to always match.

        Returns:

          bool: Boolean indicating whether the resource object matches the
            filter arguments.
        """
        if filter_args is not None:
            for prop_name in filter_args:
                prop_match = filter_args[prop_name]
                if not self._matches_prop(obj, prop_name, prop_match):
                    return False
        return True

    def _matches_prop(self, obj, prop_name, prop_match):
        if isinstance(prop_match, (list, tuple)):
            for pm in prop_match:
                if not self._matches_prop(obj, prop_name, pm):
                    return False
        else:
            # TODO: Here, we could match by regexp.
            if obj.get_property(prop_name) != prop_match:
                return False
        return True

    @property
    def resource_class(self):
        """
        The Python class of the parent resource of this manager.
        """
        return self._resource_class

    @property
    def session(self):
        """
        :class:`~zhmcclient.Session`:
          Session with the HMC.
        """
        if self._session is None:
            raise AssertionError("%s.session: No session set (in top-level "
                                 "resource manager class?)" %
                                 self.__class__.__name__)
        return self._session

    @property
    def parent(self):
        """
        Subclass of :class:`~zhmcclient.BaseResource`:
          Parent resource defining the scope for this manager.

          `None`, if the manager has no parent, i.e. when it manages top-level
          resources.
        """
        return self._parent

    def list(self, full_properties=False, filter_args=None):
        """
        Interface for a method that lists resources, and that needs to be
        implemented by resource manager classes derived from this base class.

        Authorization requirements:

        * see the `list()` method in the derived classes.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only a minimal set as returned by the list
            operation.

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          List of resource objects managed by this manager. The resource
          objects have a set of properties according to the `full_properties`
          parameter.

        Raises:

          : Exceptions raised by the `list()` methods in derived resource
            manager classes (see :ref:`Resources`).
        """
        raise NotImplementedError

    @logged_api_call
    def find(self, **filter_args):
        """
        Find exactly one resource that is managed by this manager, by matching
        their resource properties against the specified filter arguments, and
        return its Python resource object (e.g. for a CPC,
        :class:`~zhmcclient.Cpc` object is returned).

        If only the 'name' resource property is specified, an optimized lookup
        is performed that uses a name-to-URI mapping cached in this manager
        object.

        Authorization requirements:

        * see the `list()` method in the derived classes.

        Keyword Arguments:

          : The keyword arguments are used as filter arguments that narrow the
            list of returned resources to those that match the specified filter
            arguments. For details, see :ref:`Filtering`.

            Note that some resource property names are not valid as Python
            parameter names (e.g. "adapter-family"). Such resource properties
            can still be used for filtering, but must be specified by the
            caller via a parameter dictionary (see the example for details).

        Returns:

          Resource object, if found. The resource object has a minimal set of
          properties.

        Raises:

          :exc:`~zhmcclient.NotFound`: No matching resource found.
          :exc:`~zhmcclient.NoUniqueMatch`: More than one matching resource
            found.
          : Exceptions raised by the `list()` methods in derived resource
            manager classes (see :ref:`Resources`).

        Example:

          The following example finds a CPC by its name. Because the 'name'
          resource property is also a valid Python variable name, there are two
          ways for the caller to specify the filter arguments for this method:

          * As named parameters::

              cpc = client.cpcs.find(name='CPC001')

          * As a parameter dictionary::

              filter_args = {'name': 'CPC0001'}
              cpc = client.cpcs.find(**filter_args)
        """
        obj_list = self.findall(**filter_args)
        num_objs = len(obj_list)
        if num_objs == 0:
            raise NotFound
        elif num_objs > 1:
            raise NoUniqueMatch
        else:
            return obj_list[0]

    @logged_api_call
    def findall(self, **filter_args):
        """
        Find zero or more resources that are managed by this manager, by
        matching their resource properties against the specified filter
        arguments, and return a list of their Python resource objects (e.g. for
        CPCs, a list of :class:`~zhmcclient.Cpc` objects is returned).

        If only the 'name' resource property is specified, an optimized lookup
        is performed that uses a name-to-URI mapping cached in this manager
        object.

        Authorization requirements:

        * see the `list()` method in the derived classes.

        Keyword Arguments:

          : The keyword arguments are used as filter arguments that narrow the
            list of returned resources to those that match the specified filter
            arguments. For details, see :ref:`Filtering`.

            Note that some resource property names are not valid as Python
            parameter names (e.g. "adapter-family"). Such resource properties
            can still be used for this method, but must be specified via a
            parameter dictionary (see the example for details).

        Returns:

          A list of zero or more resource objects that were found. The resource
          objects have a minimal set of properties.

        Raises:

          : Exceptions raised by the `list()` methods in derived resource
            manager classes (see :ref:`Resources`).

        Example:

          The following example finds adapters of the OSA family in a CPC.
          Because the resource property for the adapter family is named
          'adapter-family', it is not suitable as a Python variable name.
          Therefore, the only way for the caller to specify it is via a
          parameter dictionary::

              filter_args = {'adapter-family': 'osa'}
              osa_adapters = cpc.adapters.findall(**filter_args)
        """
        if len(filter_args) == 1 and self._name_prop in filter_args:
            try:
                obj = self.find_by_name(filter_args[self._name_prop])
            except NotFound:
                return []
            return [obj]
        else:
            obj_list = self.list(filter_args=filter_args)
            return obj_list

    @logged_api_call
    def find_by_name(self, name):
        """
        Find a resource by name (i.e. value of its 'name' resource property)
        and return its Python resource object (e.g. for a CPC, a
        :class:`~zhmcclient.Cpc` object is returned).

        This method performs an optimized lookup that uses a name-to-URI
        mapping cached in this manager object.

        This method is automatically used by the
        :meth:`~zhmcclient.BaseManager.find` and
        :meth:`~zhmcclient.BaseManager.findall` methods, so it does not
        normally need to be used directly by users.

        Authorization requirements:

        * see the `list()` method in the derived classes.

        Parameters:

          name (string):
            Name of the resource (value of its 'name' resource property).

        Returns:

          Resource object, if found. The resource object has a minimal set of
          properties.

        Raises:

          :exc:`~zhmcclient.NotFound`: No matching resource found.
          : Exceptions raised by the `list()` methods in derived resource
            manager classes (see :ref:`Resources`).

        Example:

          The following example finds a CPC by its name::

              cpc = client.cpcs.find_by_name('CPC001')
        """
        uri = self._get_uri(name)
        obj = self.resource_class(
            manager=self,
            uri=uri,
            name=name,
            properties=None)
        return obj

    @logged_api_call
    def flush(self):
        """
        Flush the cached name-to-URI mapping.

        This only needs to be done after renaming a resource.
        """
        self._uris.clear()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`Virtual Function` is a logical entity that provides a
:term:`Partition` with access to
:term:`Accelerator Adapters <Accelerator Adapter>`.

Virtual Function resources are contained in Partition resources.

Virtual Functions only exist in :term:`CPCs <CPC>` that are in DPM mode.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._logging import get_logger, logged_api_call

__all__ = ['VirtualFunctionManager', 'VirtualFunction']

LOG = get_logger(__name__)


class VirtualFunctionManager(BaseManager):
    """
    Manager providing access to the
    :term:`Virtual Functions <Virtual Function>` in a particular
    :term:`Partition`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible via the following instance variable of a
    :class:`~zhmcclient.Partition` object (in DPM mode):

    * :attr:`~zhmcclient.Partition.virtual_functions`
    """

    def __init__(self, partition):
        # This function should not go into the docs.
        # Parameters:
        #   partition (:class:`~zhmcclient.Partition`):
        #     Partition defining the scope for this manager.
        super(VirtualFunctionManager, self).__init__(
            resource_class=VirtualFunction,
            parent=partition,
            uri_prop='element-uri',
            name_prop='name',
            query_props=[])

    @property
    def partition(self):
        """
        :class:`~zhmcclient.Partition`: :term:`Partition` defining the scope
        for this manager.
        """
        return self._parent

    @logged_api_call
    def list(self, full_properties=False, filter_args=None):
        """
        List the Virtual Functions of this Partition.

        Authorization requirements:

        * Object-access permission to this Partition.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          : A list of :class:`~zhmcclient.VirtualFunction` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        resource_obj_list = []
        uris = self.partition.get_property('virtual-function-uris')
        if uris:
            for uri in uris:

                resource_obj = self.resource_class(
                    manager=self,
                    uri=uri,
                    name=None,
                    properties=None)

                if self._matches_filters(resource_obj, filter_args):
                    resource_obj_list.append(resource_obj)
                    if full_properties:
                        resource_obj.pull_full_properties()
        return resource_obj_list

    @logged_api_call
    def create(self, properties):
        """
        Create a Virtual Function in this Partition.

        Authorization requirements:

        * Object-access permission to this Partition.
        * Object-access permission to the backing accelerator Adapter.
        * Task permission for the "Partition Details" task.

        Parameters:

          properties (dict): Initial property values.
            Allowable properties are defined in section 'Request body contents'
            in section 'Create Virtual Function' in the :term:`HMC API` book.

        Returns:

          VirtualFunction:
            The resource object for the new Virtual Function.
            The object will have its 'element-uri' property set as returned by
            the HMC, and will also have the input properties set.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        result = self.session.post(self.partition.uri + '/virtual-functions',
                                   body=properties)
        # There should not be overlaps, but just in case there are, the
        # returned props should overwrite the input props:
        props = properties.copy()
        props.update(result)
        return VirtualFunction(self, props['element-uri'], None, props)


class VirtualFunction(BaseResource):
    """
    Representation of a :term:`Virtual Function`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    For the properties of a Virtual Function, see section
    'Data model - Virtual Function Element Object' in section
    'Partition object' in the :term:`HMC API` book.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.VirtualFunctionManager`).
    """

    def __init__(self, manager, uri, name=None, properties=None):
        # This function should not go into the docs.
        #   manager (:class:`~zhmcclient.VirtualFunctionManager`):
        #     Manager object for this resource object.
        #   uri (string):
        #     Canonical URI path of the resource.
        #   name (string):
        #     Name of the resource.
        #   properties (dict):
        #     Properties to be set for this resource object. May be `None` or
        #     empty.
        if not isinstance(manager, VirtualFunctionManager):
            raise AssertionError("VirtualFunction init: Expected manager "
                                 "type %s, got %s" %
                                 (VirtualFunctionManager, type(manager)))
        super(VirtualFunction, self).__init__(manager, uri, name, properties)

    @logged_api_call
    def delete(self):
        """
        Delete this Virtual Function.

        Authorization requirements:

        * Object-access permission to the Partition of this Virtual Function.
        * Task permission for the "Partition Details" task.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.delete(self._uri)

    @logged_api_call
    def update_properties(self, properties):
        """
        Update writeable properties of this Virtual Function.

        Authorization requirements:

        * Object-access permission to the Partition of this Virtual Function.
        * When updating the "adapter-uri" property, object-access permission to
          the Adapter identified in that URI.
        * Task permission for the "Partition Details" task.

        Parameters:

          properties (dict): New values for the properties to be updated.
            Properties not to be updated are omitted.
            Allowable properties are the properties with qualifier (w) in
            section 'Data model - Virtual Function element object' in the
            :term:`HMC API` book.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.manager.session.post(self._uri, body=properties)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _virtual_switch module.
"""

from __future__ import absolute_import, print_function

import unittest
import re
import requests_mock

from zhmcclient import Session, Client, Nic


class VirtualSwitchTests(unittest.TestCase):
    """All tests for VirtualSwitch and VirtualSwitchManager classes."""

    def setUp(self):
        self.session = Session('vswitch-dpm-host', 'vswitch-user',
                               'vswitch-pwd')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'vswitch-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/vswitch-cpc-id-1',
                        'name': 'CPC',
                        'status': 'service-required',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)

            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on VirtualSwitchManager instance in CPC."""
        vswitch_mgr = self.cpc.virtual_switches
        self.assertEqual(vswitch_mgr.cpc, self.cpc)

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties
        on VirtualSwitchManager instance in CPC.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }

            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)

            self.assertEqual(len(vswitches), len(result['virtual-switches']))
            for idx, vswitch in enumerate(vswitches):
                self.assertEqual(
                    vswitch.properties,
                    result['virtual-switches'][idx])
                self.assertEqual(
                    vswitch.uri,
                    result['virtual-switches'][idx]['object-uri'])
                self.assertFalse(vswitch.full_properties)
                self.assertEqual(vswitch.manager, vswitch_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on
        VirtualSwitchManager instance in CPC.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hipersockets'
                    }
                ]
            }

            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            mock_result_virtual_switch1 = {
                'name': 'VSWITCH1',
                'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                'type': 'osd',
                'class': 'virtual-switch',
                'description': 'Test VirtualSwitch',
                'more_properties': 'bliblablub'
            }
            m.get('/api/virtual-switches/fake-vswitch-id1',
                  json=mock_result_virtual_switch1)
            mock_result_virtual_switch2 = {
                'name': 'VSWITCH2',
                'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                'type': 'hipersockets',
                'class': 'virtual-switch',
                'description': 'Test VirtualSwitch',
                'more_properties': 'bliblablub'
            }
            m.get('/api/virtual-switches/fake-vswitch-id2',
                  json=mock_result_virtual_switch2)

            vswitches = vswitch_mgr.list(full_properties=True)

            self.assertEqual(len(vswitches), len(result['virtual-switches']))
            for idx, vswitch in enumerate(vswitches):
                self.assertEqual(vswitch.properties['name'],
                                 result['virtual-switches'][idx]['name'])
                self.assertEqual(
                    vswitch.uri,
                    result['virtual-switches'][idx]['object-uri'])
                self.assertTrue(vswitch.full_properties)
                self.assertEqual(vswitch.manager, vswitch_mgr)

    def test_update_properties(self):
        """
        This tests the 'Update VirtualSwitch Properties' operation.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }
            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)
            vswitch = vswitches[0]
            m.post(
                "/api/virtual-switches/fake-vswitch-id1",
                json=result)
            status = vswitch.update_properties(properties={})
            self.assertEqual(status, None)

    def test_get_connected_nics(self):
        """
        This tests the `get_connected_nics()` method.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }
            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)
            vswitch = vswitches[0]
            result = {
                'connected-vnic-uris': [
                    '/api/partitions/fake-part-id1/nics/fake-nic-id1',
                    '/api/partitions/fake-part-id1/nics/fake-nic-id2',
                    '/api/partitions/fake-part-id1/nics/fake-nic-id3'
                ]
            }

            m.get(
                "/api/virtual-switches/fake-vswitch-id1/"
                "operations/get-connected-vnics",
                json=result)

            nics = vswitch.get_connected_nics()

            self.assertTrue(isinstance(nics, list))
            for i, nic in enumerate(nics):
                self.assertTrue(isinstance(nic, Nic))
                nic_uri = result['connected-vnic-uris'][i]
                self.assertEqual(nic.uri, nic_uri)
                self.assertEqual(nic.properties['element-uri'], nic_uri)
                m = re.match(r"^/api/partitions/([^/]+)/nics/([^/]+)/?$",
                             nic_uri)
                nic_id = m.group(2)
                self.assertEqual(nic.properties['element-id'], nic_id)


if __name__ == '__main__':
    unittest.main()
